# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from typing import Any, Dict, List, Optional

import pydantic

from stytch.b2b.models.organizations import (
    CreateResponse,
    DeleteMemberResponse,
    DeleteResponse,
    GetResponse,
    SearchMembersResponse,
    SearchResponse,
    UpdateResponse,
)
from stytch.core.api_base import ApiBase
from stytch.core.http.client import AsyncClient, SyncClient
from stytch.core.models import SearchQuery


class Organizations:
    def __init__(
        self,
        api_base: ApiBase,
        sync_client: SyncClient,
        async_client: AsyncClient,
    ) -> None:
        self.api_base = api_base
        self.sync_client = sync_client
        self.async_client = async_client

    @property
    def sub_url(self) -> str:
        return "organizations"

    def create(
        self,
        organization_name: str,
        organization_slug: str,
        trusted_metadata: Optional[Dict[str, Any]] = None,
        email_jit_provisioning: Optional[str] = None,
        email_invites: Optional[str] = None,
        email_allowed_domains: Optional[List[str]] = None,
        sso_jit_provisioning: Optional[str] = None,
    ) -> CreateResponse:
        """Creates a new Organization. It requires a name and a unique slug.

        See the Organization authentication settings resource to learn more about fields like email_jit_provisioning, email_allowed_domains, sso_jit_provisioning, etc., and their behaviors.

        Parameters:

        - `organization_name`: The name of the Organization.

        - `organization_slug`: The unique URL slug of the Organization.

        - `organzation_logo_url`: The image URL of the Organization’s logo.

        - `trusted_metadata`: An arbitrary JSON object for storing application-specific data.

        - `email_jit_provisioning`: The setting that controls the JIT provisioning of new Members when authenticating via email. The accepted values are:

          - RESTRICTED – only new Members with verified emails that comply with email_allowed_domains can be provisioned upon authentication.
          - NOT_ALLOWED – disable JIT provisioning.

        - `email_invites`: The setting that controls how a new Member can be invited to an organization by email. The accepted values are:

          - ALL_ALLOWED – any new Member can be invited to join
          - RESTRICTED – only new Members with verified emails that comply with email_allowed_domains can be invited
          - NOT_ALLOWED – disable invites

        - `email_allowed_domains`: An array of email domains that allow invitations or JIT provisioning for new Members. This list is enforced when either email_invites or email_jit_provisioning is set to RESTRICTED.

        - `sso_jit_provisioning`: The setting that controls the JIT provisioning of Members when authenticating via SSO. The accepted values are:
          - ALL_ALLOWED – any new Member can be provisioned upon authentication
          - RESTRICTED – only new Members with SSO logins that comply with sso_jit_provisioning_allowed_connections can be provisioned upon authentication
          - NOT_ALLOWED – disable JIT provisioning
        """  # noqa

        payload: Dict[str, Any] = {
            "organization_name": organization_name,
            "organization_slug": organization_slug,
        }

        if trusted_metadata is not None:
            payload["trusted_metadata"] = trusted_metadata
        if email_jit_provisioning is not None:
            payload["email_jit_provisioning"] = email_jit_provisioning
        if email_invites is not None:
            payload["email_invites"] = email_invites
        if email_allowed_domains is not None:
            payload["email_allowed_domains"] = email_allowed_domains
        if sso_jit_provisioning is not None:
            payload["sso_jit_provisioning"] = sso_jit_provisioning

        url = self.api_base.route_with_sub_url(self.sub_url, None)

        res = self.sync_client.post(url, json=payload)
        return CreateResponse.from_json(res.response.status_code, res.json)

    async def create_async(
        self,
        organization_name: str,
        organization_slug: str,
        trusted_metadata: Optional[Dict[str, Any]] = None,
        email_jit_provisioning: Optional[str] = None,
        email_invites: Optional[str] = None,
        email_allowed_domains: Optional[List[str]] = None,
        sso_jit_provisioning: Optional[str] = None,
    ) -> CreateResponse:
        """Creates a new Organization. It requires a name and a unique slug.

        See the Organization authentication settings resource to learn more about fields like email_jit_provisioning, email_allowed_domains, sso_jit_provisioning, etc., and their behaviors.

        Parameters:

        - `organization_name`: The name of the Organization.

        - `organization_slug`: The unique URL slug of the Organization.

        - `organzation_logo_url`: The image URL of the Organization’s logo.

        - `trusted_metadata`: An arbitrary JSON object for storing application-specific data.

        - `email_jit_provisioning`: The setting that controls the JIT provisioning of new Members when authenticating via email. The accepted values are:

          - RESTRICTED – only new Members with verified emails that comply with email_allowed_domains can be provisioned upon authentication.
          - NOT_ALLOWED – disable JIT provisioning.

        - `email_invites`: The setting that controls how a new Member can be invited to an organization by email. The accepted values are:

          - ALL_ALLOWED – any new Member can be invited to join
          - RESTRICTED – only new Members with verified emails that comply with email_allowed_domains can be invited
          - NOT_ALLOWED – disable invites

        - `email_allowed_domains`: An array of email domains that allow invitations or JIT provisioning for new Members. This list is enforced when either email_invites or email_jit_provisioning is set to RESTRICTED.

        - `sso_jit_provisioning`: The setting that controls the JIT provisioning of Members when authenticating via SSO. The accepted values are:
          - ALL_ALLOWED – any new Member can be provisioned upon authentication
          - RESTRICTED – only new Members with SSO logins that comply with sso_jit_provisioning_allowed_connections can be provisioned upon authentication
          - NOT_ALLOWED – disable JIT provisioning
        """  # noqa

        payload: Dict[str, Any] = {
            "organization_name": organization_name,
            "organization_slug": organization_slug,
        }

        if trusted_metadata is not None:
            payload["trusted_metadata"] = trusted_metadata
        if email_jit_provisioning is not None:
            payload["email_jit_provisioning"] = email_jit_provisioning
        if email_invites is not None:
            payload["email_invites"] = email_invites
        if email_allowed_domains is not None:
            payload["email_allowed_domains"] = email_allowed_domains
        if sso_jit_provisioning is not None:
            payload["sso_jit_provisioning"] = sso_jit_provisioning

        url = self.api_base.route_with_sub_url(self.sub_url, None)

        res = await self.async_client.post(url, json=payload)
        return CreateResponse.from_json(res.response.status, res.json)

    def get(
        self,
        organization_id: str,
    ) -> GetResponse:
        """Returns an Organization specified by organization_id.

        Parameters:

        - `organization_id`: Globally unique UUID that identifies a specific Organization in the Stytch API. The organization_id is critical to perform operations on an Organization in the Stytch API, so be sure to preserve this value.
        """  # noqa

        payload: Dict[str, Any] = {
            "organization_id": organization_id,
        }

        url = self.api_base.route_with_sub_url(self.sub_url, organization_id)

        res = self.sync_client.get(url, params=payload)
        return GetResponse.from_json(res.response.status_code, res.json)

    async def get_async(
        self,
        organization_id: str,
    ) -> GetResponse:
        """Returns an Organization specified by organization_id.

        Parameters:

        - `organization_id`: Globally unique UUID that identifies a specific Organization in the Stytch API. The organization_id is critical to perform operations on an Organization in the Stytch API, so be sure to preserve this value.
        """  # noqa

        payload: Dict[str, Any] = {
            "organization_id": organization_id,
        }

        url = self.api_base.route_with_sub_url(self.sub_url, organization_id)

        res = await self.async_client.get(url, params=payload)
        return GetResponse.from_json(res.response.status, res.json)

    def update(
        self,
        organization_id: str,
        organization_name: Optional[str] = None,
        organization_slug: Optional[str] = None,
        organization_logo_url: Optional[str] = None,
        trusted_metadata: Optional[Dict[str, Any]] = None,
        sso_default_connection_id: Optional[str] = None,
        sso_jit_provisioning_allowed_connections: Optional[List[str]] = None,
        sso_jit_provisioning: Optional[str] = None,
        email_allowed_domains: Optional[List[str]] = None,
        email_jit_provisioning: Optional[str] = None,
        email_invites: Optional[str] = None,
    ) -> UpdateResponse:
        """Updates an Organization specified by organization_id.

        See the Organization authentication settings resource to learn more about fields like email_jit_provisioning, email_allowed_domains, sso_jit_provisioning, etc., and their behaviors.

        Parameters:

        - `organization_id`: Globally unique UUID that identifies a specific Organization. The organization_id is critical to perform operations on an Organization, so be sure to preserve this value.

        - `organization_name`: The name of the Organization.

        - `organization_slug`: The unique URL slug of the Organization.

        - `organization_logo_url`: The image URL of the Organization’s logo.

        - `trusted_metadata`: An arbitrary JSON object for storing application-specific data.

        - `sso_default_connection_id`: The default connection used for SSO when there are multiple active connections.

        - `sso_jit_provisioning_allowed_connections`: An array of connections used for SSO when sso_jit_provisiniong is set to RESTRICTED.

        - `sso_jit_provisioning`: The setting that controls the JIT provisioning of Members when authenticating via SSO. The accepted values are:

          - ALL_ALLOWED – any new Member can be provisioned upon authentication
          - RESTRICTED – only new Members with SSO logins that comply with sso_jit_provisioning_allowed_connections can be provisioned upon authentication
          - NOT_ALLOWED – disable JIT provisioning

        - `email_allowed_domains`: An array of email domains that allow invitations or JIT provisioning for new Members. This list is enforced when either email_invites or email_jit_provisioning is set to RESTRICTED. To remove or add domains, you must pass in the full array with all its values.

        - `email_jit_provisioning`: The setting that controls the JIT provisioning of new Members when authenticating via email. The accepted values are:

          - RESTRICTED – only new Members with verified emails that comply with email_allowed_domains can be provisioned upon authentication
          - NOT_ALLOWED – disable JIT provisioning

        - `email_invites`: The setting that controls how a new Member can be invited to an Organization by email. The three accepted values are:
          - ALL_ALLOWED – any new Member can be invited to join
          - RESTRICTED – only new Members with verified emails that comply with email_allowed_domains can be invited
          - NOT_ALLOWED – disable invites
        """  # noqa

        payload: Dict[str, Any] = {
            "organization_id": organization_id,
        }

        if organization_name is not None:
            payload["organization_name"] = organization_name
        if organization_slug is not None:
            payload["organization_slug"] = organization_slug
        if organization_logo_url is not None:
            payload["organization_logo_url"] = organization_logo_url
        if trusted_metadata is not None:
            payload["trusted_metadata"] = trusted_metadata
        if sso_default_connection_id is not None:
            payload["sso_default_connection_id"] = sso_default_connection_id
        if sso_jit_provisioning_allowed_connections is not None:
            payload[
                "sso_jit_provisioning_allowed_connections"
            ] = sso_jit_provisioning_allowed_connections
        if sso_jit_provisioning is not None:
            payload["sso_jit_provisioning"] = sso_jit_provisioning
        if email_allowed_domains is not None:
            payload["email_allowed_domains"] = email_allowed_domains
        if email_jit_provisioning is not None:
            payload["email_jit_provisioning"] = email_jit_provisioning
        if email_invites is not None:
            payload["email_invites"] = email_invites

        url = self.api_base.route_with_sub_url(self.sub_url, organization_id)

        res = self.sync_client.put(url, json=payload)
        return UpdateResponse.from_json(res.response.status_code, res.json)

    async def update_async(
        self,
        organization_id: str,
        organization_name: Optional[str] = None,
        organization_slug: Optional[str] = None,
        organization_logo_url: Optional[str] = None,
        trusted_metadata: Optional[Dict[str, Any]] = None,
        sso_default_connection_id: Optional[str] = None,
        sso_jit_provisioning_allowed_connections: Optional[List[str]] = None,
        sso_jit_provisioning: Optional[str] = None,
        email_allowed_domains: Optional[List[str]] = None,
        email_jit_provisioning: Optional[str] = None,
        email_invites: Optional[str] = None,
    ) -> UpdateResponse:
        """Updates an Organization specified by organization_id.

        See the Organization authentication settings resource to learn more about fields like email_jit_provisioning, email_allowed_domains, sso_jit_provisioning, etc., and their behaviors.

        Parameters:

        - `organization_id`: Globally unique UUID that identifies a specific Organization. The organization_id is critical to perform operations on an Organization, so be sure to preserve this value.

        - `organization_name`: The name of the Organization.

        - `organization_slug`: The unique URL slug of the Organization.

        - `organization_logo_url`: The image URL of the Organization’s logo.

        - `trusted_metadata`: An arbitrary JSON object for storing application-specific data.

        - `sso_default_connection_id`: The default connection used for SSO when there are multiple active connections.

        - `sso_jit_provisioning_allowed_connections`: An array of connections used for SSO when sso_jit_provisiniong is set to RESTRICTED.

        - `sso_jit_provisioning`: The setting that controls the JIT provisioning of Members when authenticating via SSO. The accepted values are:

          - ALL_ALLOWED – any new Member can be provisioned upon authentication
          - RESTRICTED – only new Members with SSO logins that comply with sso_jit_provisioning_allowed_connections can be provisioned upon authentication
          - NOT_ALLOWED – disable JIT provisioning

        - `email_allowed_domains`: An array of email domains that allow invitations or JIT provisioning for new Members. This list is enforced when either email_invites or email_jit_provisioning is set to RESTRICTED. To remove or add domains, you must pass in the full array with all its values.

        - `email_jit_provisioning`: The setting that controls the JIT provisioning of new Members when authenticating via email. The accepted values are:

          - RESTRICTED – only new Members with verified emails that comply with email_allowed_domains can be provisioned upon authentication
          - NOT_ALLOWED – disable JIT provisioning

        - `email_invites`: The setting that controls how a new Member can be invited to an Organization by email. The three accepted values are:
          - ALL_ALLOWED – any new Member can be invited to join
          - RESTRICTED – only new Members with verified emails that comply with email_allowed_domains can be invited
          - NOT_ALLOWED – disable invites
        """  # noqa

        payload: Dict[str, Any] = {
            "organization_id": organization_id,
        }

        if organization_name is not None:
            payload["organization_name"] = organization_name
        if organization_slug is not None:
            payload["organization_slug"] = organization_slug
        if organization_logo_url is not None:
            payload["organization_logo_url"] = organization_logo_url
        if trusted_metadata is not None:
            payload["trusted_metadata"] = trusted_metadata
        if sso_default_connection_id is not None:
            payload["sso_default_connection_id"] = sso_default_connection_id
        if sso_jit_provisioning_allowed_connections is not None:
            payload[
                "sso_jit_provisioning_allowed_connections"
            ] = sso_jit_provisioning_allowed_connections
        if sso_jit_provisioning is not None:
            payload["sso_jit_provisioning"] = sso_jit_provisioning
        if email_allowed_domains is not None:
            payload["email_allowed_domains"] = email_allowed_domains
        if email_jit_provisioning is not None:
            payload["email_jit_provisioning"] = email_jit_provisioning
        if email_invites is not None:
            payload["email_invites"] = email_invites

        url = self.api_base.route_with_sub_url(self.sub_url, organization_id)

        res = await self.async_client.put(url, json=payload)
        return UpdateResponse.from_json(res.response.status, res.json)

    def delete(
        self,
        organization_id: str,
    ) -> DeleteResponse:
        """Deletes an Organization, its Members, its Sessions, and other resources under it.

        Parameters:

        - `organization_id`: Globally unique UUID that identifies a specific Organization. The organization_id is critical to perform operations on an Organization, so be sure to preserve this value.
        """  # noqa

        url = self.api_base.route_with_sub_url(self.sub_url, organization_id)

        res = self.sync_client.delete(url)
        return DeleteResponse.from_json(res.response.status_code, res.json)

    async def delete_async(
        self,
        organization_id: str,
    ) -> DeleteResponse:
        """Deletes an Organization, its Members, its Sessions, and other resources under it.

        Parameters:

        - `organization_id`: Globally unique UUID that identifies a specific Organization. The organization_id is critical to perform operations on an Organization, so be sure to preserve this value.
        """  # noqa

        url = self.api_base.route_with_sub_url(self.sub_url, organization_id)

        res = await self.async_client.delete(url)
        return DeleteResponse.from_json(res.response.status, res.json)

    def delete_member(
        self,
        organization_id: str,
        member_id: str,
    ) -> DeleteMemberResponse:
        """Deletes a Member from an Organization.

        Parameters:

        - `organization_id`: Globally unique UUID that identifies a specific Organization. The organization_id is critical to perform operations on an Organization, so be sure to preserve this value.

        - `member_id`: Globally unique UUID that identifies a specific Member in the Stytch API. The member_id is critical to perform operations on a Member, so be sure to preserve this value.
        """  # noqa

        url = self.api_base.route_with_sub_url(
            self.sub_url, f"{organization_id}/member/{member_id}"
        )

        res = self.sync_client.delete(url)
        return DeleteMemberResponse.from_json(res.response.status_code, res.json)

    async def delete_member_async(
        self,
        organization_id: str,
        member_id: str,
    ) -> DeleteMemberResponse:
        """Deletes a Member from an Organization.

        Parameters:

        - `organization_id`: Globally unique UUID that identifies a specific Organization. The organization_id is critical to perform operations on an Organization, so be sure to preserve this value.

        - `member_id`: Globally unique UUID that identifies a specific Member in the Stytch API. The member_id is critical to perform operations on a Member, so be sure to preserve this value.
        """  # noqa

        url = self.api_base.route_with_sub_url(
            self.sub_url, f"{organization_id}/member/{member_id}"
        )

        res = await self.async_client.delete(url)
        return DeleteMemberResponse.from_json(res.response.status, res.json)

    def search(
        self,
        cursor: Optional[str] = None,
        limit: Optional[int] = None,
        query: Optional[SearchQuery] = None,
    ) -> SearchResponse:
        """Search for Organizations. If you send a request with no body params, no filtering will be applied and the endpoint will return all Organizations.

        All fuzzy search filters require a minimum of at least 3 characters

        Parameters:

        - `cursor`: The cursor field allows you to paginate through your results. Each result array is limited to 1000 results, if your query returns more than 1000 results, you will need to paginate the responses using the cursor. If you receive a response that includes a non-null next_cursor in the results_metadata object, you should repeat the call, being sure to include all of the original fields, but pass in the next_cursor in the cursor field. Continue to make calls until the next_cursor in the response is null.

        - `limit`: The number of Organizations to return per page, the default is 100. A maximum of 1000 Organizations can be returned by a single request. If the total size of your result is greater than one page size, you must paginate the response. See the cursor field above.

        - `query`: The optional query object contains the operator, e.g. AND or OR, and the operands that will filter your Organizations. Only an operator is required, if you include no operands, no filtering will be applied. Similarly if you include no query object, no filtering is applied and we'll return all of your Organizations with no filtering applied.
        """  # noqa

        payload: Dict[str, Any] = {}

        if cursor is not None:
            payload["cursor"] = cursor
        if limit is not None:
            payload["limit"] = limit
        if query is not None:
            payload["query"] = query

        url = self.api_base.route_with_sub_url(self.sub_url, "search")

        res = self.sync_client.post(url, json=payload)
        return SearchResponse.from_json(res.response.status_code, res.json)

    async def search_async(
        self,
        cursor: Optional[str] = None,
        limit: Optional[int] = None,
        query: Optional[SearchQuery] = None,
    ) -> SearchResponse:
        """Search for Organizations. If you send a request with no body params, no filtering will be applied and the endpoint will return all Organizations.

        All fuzzy search filters require a minimum of at least 3 characters

        Parameters:

        - `cursor`: The cursor field allows you to paginate through your results. Each result array is limited to 1000 results, if your query returns more than 1000 results, you will need to paginate the responses using the cursor. If you receive a response that includes a non-null next_cursor in the results_metadata object, you should repeat the call, being sure to include all of the original fields, but pass in the next_cursor in the cursor field. Continue to make calls until the next_cursor in the response is null.

        - `limit`: The number of Organizations to return per page, the default is 100. A maximum of 1000 Organizations can be returned by a single request. If the total size of your result is greater than one page size, you must paginate the response. See the cursor field above.

        - `query`: The optional query object contains the operator, e.g. AND or OR, and the operands that will filter your Organizations. Only an operator is required, if you include no operands, no filtering will be applied. Similarly if you include no query object, no filtering is applied and we'll return all of your Organizations with no filtering applied.
        """  # noqa

        payload: Dict[str, Any] = {}

        if cursor is not None:
            payload["cursor"] = cursor
        if limit is not None:
            payload["limit"] = limit
        if query is not None:
            payload["query"] = query

        url = self.api_base.route_with_sub_url(self.sub_url, "search")

        res = await self.async_client.post(url, json=payload)
        return SearchResponse.from_json(res.response.status, res.json)

    def search_members(
        self,
        organization_ids: List[str],
        cursor: Optional[str] = None,
        limit: Optional[int] = None,
        query: Optional[SearchQuery] = None,
    ) -> SearchMembersResponse:
        """Search for members within an Organization. It requires an organization_id. Submitting an empty query returns all Members of an Organization.

        Parameters:

        - `organization_ids`: List of globally unique UUIDs that identify specific Organizations. The organization_id is critical to perform operations on an Organization, so be sure to preserve this value.

        - `cursor`: The cursor field allows you to paginate through your results. Each result array is limited to 1000 results, if your query returns more than 1000 results, you will need to paginate the responses using the cursor. If you receive a response that includes a non-null next_cursor in the results_metadata object, you should repeat the call, being sure to include all of the original fields, but pass in the next_cursor in the cursor field. Continue to make calls until the next_cursor in the response is null.

        - `limit`: The number of Organizations to return per page, the default is 100. A maximum of 1000 Members can be returned by a single request. If the total size of your result is greater than one page size, you must paginate the response. See the cursor field above.

        - `query`: The optional query object contains the operator, e.g. AND or OR, and the operands that will filter your Members. Only an operator is required, if you include no operands, no filtering will be applied. Similarly if you include no query object, no filtering is applied and we'll return all of your Members with no filtering applied.
        """  # noqa

        payload: Dict[str, Any] = {
            "organization_ids": organization_ids,
        }

        if cursor is not None:
            payload["cursor"] = cursor
        if limit is not None:
            payload["limit"] = limit
        if query is not None:
            payload["query"] = (
                query.dict() if isinstance(query, pydantic.BaseModel) else query
            )

        url = self.api_base.route_with_sub_url(self.sub_url, "members/search")

        res = self.sync_client.post(url, json=payload)
        return SearchMembersResponse.from_json(res.response.status_code, res.json)

    async def search_members_async(
        self,
        organization_ids: List[str],
        cursor: Optional[str] = None,
        limit: Optional[int] = None,
        query: Optional[SearchQuery] = None,
    ) -> SearchMembersResponse:
        """Search for members within an Organization. It requires an organization_id. Submitting an empty query returns all Members of an Organization.

        Parameters:

        - `organization_ids`: List of globally unique UUIDs that identify specific Organizations. The organization_id is critical to perform operations on an Organization, so be sure to preserve this value.

        - `cursor`: The cursor field allows you to paginate through your results. Each result array is limited to 1000 results, if your query returns more than 1000 results, you will need to paginate the responses using the cursor. If you receive a response that includes a non-null next_cursor in the results_metadata object, you should repeat the call, being sure to include all of the original fields, but pass in the next_cursor in the cursor field. Continue to make calls until the next_cursor in the response is null.

        - `limit`: The number of Organizations to return per page, the default is 100. A maximum of 1000 Members can be returned by a single request. If the total size of your result is greater than one page size, you must paginate the response. See the cursor field above.

        - `query`: The optional query object contains the operator, e.g. AND or OR, and the operands that will filter your Members. Only an operator is required, if you include no operands, no filtering will be applied. Similarly if you include no query object, no filtering is applied and we'll return all of your Members with no filtering applied.
        """  # noqa

        payload: Dict[str, Any] = {
            "organization_ids": organization_ids,
        }

        if cursor is not None:
            payload["cursor"] = cursor
        if limit is not None:
            payload["limit"] = limit
        if query is not None:
            payload["query"] = (
                query.dict() if isinstance(query, pydantic.BaseModel) else query
            )

        url = self.api_base.route_with_sub_url(self.sub_url, "members/search")

        res = await self.async_client.post(url, json=payload)
        return SearchMembersResponse.from_json(res.response.status, res.json)



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from typing import Any, Dict, Optional

from stytch.b2b.api.magic_links_email import Email
from stytch.b2b.models.magic_links import AuthenticateResponse
from stytch.core.api_base import ApiBase
from stytch.core.http.client import AsyncClient, SyncClient


class MagicLinks:
    def __init__(
        self,
        api_base: ApiBase,
        sync_client: SyncClient,
        async_client: AsyncClient,
    ) -> None:
        self.api_base = api_base
        self.sync_client = sync_client
        self.async_client = async_client
        self.email = Email(api_base, sync_client, async_client)

    @property
    def sub_url(self) -> str:
        return "magic_links"

    def authenticate(
        self,
        magic_links_token: str,
        pkce_code_verifier: Optional[str] = None,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ) -> AuthenticateResponse:
        """Authenticate a Member with a magic link. This endpoint requires a magic link token to validate, making sure it's not expired or previously used. If the Member’s status is invited or pending, they will be updated to active.

        Parameters:

        - `magic_links_token`: The token to authenticate.

        - `pkce_code_verifier`: A base64url encoded one-time secret used to validate that the request starts and ends on the same device.

        - `session_token`: A secret token for a given Stytch Session. Read more about session_token in our Session Management guide.

        - `session_jwt`: The JSON Web Token (JWT) for a given Stytch Session. Read more about session_token in our Session Management guide.

        - `session_duration_minutes`: The Session lifetime of this many minutes from now; minimum of 5 and a maximum of 129600 minutes (90 days). Note that a successful authentication will continue to extend the Session this many minutes.

        - `session_custom_claims`: Add a custom claims map to the Session being authenticated. Claims are only created if a Session is initialized by providing a value in Session duration minutes. Claims will be included on the Session object and in the JWT. To update a key in an existing Session, supply a new value. To delete a key, supply a null value.
          Custom claims made with reserved claims ("iss", "sub", "aud", "exp", "nbf", "iat", "jti") will be ignored. Total custom claims size cannot exceed four kilobytes
        """  # noqa

        payload: Dict[str, Any] = {
            "magic_links_token": magic_links_token,
        }

        if pkce_code_verifier is not None:
            payload["pkce_code_verifier"] = pkce_code_verifier
        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt
        if session_duration_minutes is not None:
            payload["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims is not None:
            payload["session_custom_claims"] = session_custom_claims

        url = self.api_base.route_with_sub_url(self.sub_url, "authenticate")

        res = self.sync_client.post(url, json=payload)
        return AuthenticateResponse.from_json(res.response.status_code, res.json)

    async def authenticate_async(
        self,
        magic_links_token: str,
        pkce_code_verifier: Optional[str] = None,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ) -> AuthenticateResponse:
        """Authenticate a Member with a magic link. This endpoint requires a magic link token to validate, making sure it's not expired or previously used. If the Member’s status is invited or pending, they will be updated to active.

        Parameters:

        - `magic_links_token`: The token to authenticate.

        - `pkce_code_verifier`: A base64url encoded one-time secret used to validate that the request starts and ends on the same device.

        - `session_token`: A secret token for a given Stytch Session. Read more about session_token in our Session Management guide.

        - `session_jwt`: The JSON Web Token (JWT) for a given Stytch Session. Read more about session_token in our Session Management guide.

        - `session_duration_minutes`: The Session lifetime of this many minutes from now; minimum of 5 and a maximum of 129600 minutes (90 days). Note that a successful authentication will continue to extend the Session this many minutes.

        - `session_custom_claims`: Add a custom claims map to the Session being authenticated. Claims are only created if a Session is initialized by providing a value in Session duration minutes. Claims will be included on the Session object and in the JWT. To update a key in an existing Session, supply a new value. To delete a key, supply a null value.
          Custom claims made with reserved claims ("iss", "sub", "aud", "exp", "nbf", "iat", "jti") will be ignored. Total custom claims size cannot exceed four kilobytes
        """  # noqa

        payload: Dict[str, Any] = {
            "magic_links_token": magic_links_token,
        }

        if pkce_code_verifier is not None:
            payload["pkce_code_verifier"] = pkce_code_verifier
        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt
        if session_duration_minutes is not None:
            payload["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims is not None:
            payload["session_custom_claims"] = session_custom_claims

        url = self.api_base.route_with_sub_url(self.sub_url, "authenticate")

        res = await self.async_client.post(url, json=payload)
        return AuthenticateResponse.from_json(res.response.status, res.json)



#!/usr/bin/env python3

from typing import Optional

from stytch.core.models import ResponseBase, StytchSession, User


class CreateResponse(ResponseBase):
    """[Stytch docs](https://stytch.com/docs/api/create-magic-link)

    Response fields beyond those defined in `ResponseBase`:

    - `user_id`: Globally unique UUID that identifies a specific user in the Stytch API. The `user_id` critical to perform operations on a user in our API, like Get user, Delete user, etc, so be sure to preserve this value.

    - `token`: The magic link token that you'll include in your contact method of choice, e.g. email or SMS. Check out our [Embeddable magic link guide](https://stytch.com/docs/api/magic-links#embeddable-magic-links_embeddable-overview) for more detail on where to include the token.
    """  # noqa

    status_code: int
    request_id: str
    user_id: str
    token: str


class AuthenticateResponse(ResponseBase):
    """[Stytch docs](https://stytch.com/docs/api/authenticate-magic-link)

    Response fields beyond those defined in `ResponseBase`:

    - `user_id`: Globally unique UUID that identifies a specific user in the Stytch API. The `user_id` critical to perform operations on a user in our API, like Get user, Delete user, etc, so be sure to preserve this value.

    - `user`: The `user` object affected by this API call. See the [Get user endpoint](https://stytch.com/docs/api/get-user) for complete response field detail.

    - `reset_sessions`: Indicates if all other of this user's sessions need to be reset. You should check this field if you aren't using Stytch's Session product. If you are using Stytch's Session product, we revoke the user's other sessions for you.

    - `session_jwt`: The JSON Web Token (JWT) for a given Stytch session. Read more about JWTs in our [session management guide](https://stytch.com/docs/sessions#using-sessions).

    - `session_token`: A secret token for a given Stytch session. Read more about `session_token` in our [session management guide](https://stytch.com/docs/sessions#using-sessions).

    - `session`: If you initiate a session, by including `session_duration_minutes` in your authenticate call, you'll receive a full session object in the response.
      See [GET sessions](https://stytch.com/docs/api/session-get) for complete response fields.
    """  # noqa

    status_code: int
    request_id: str
    user_id: str
    user: User
    method_id: str
    reset_sessions: bool
    session_jwt: str
    session_token: str
    session: Optional[StytchSession]



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from typing import Any, Dict, Optional

from stytch.core.api_base import ApiBase
from stytch.core.http.client import AsyncClient, SyncClient
from stytch.models.passwords_existing_password import ResetResponse


class ExistingPassword:
    def __init__(
        self,
        api_base: ApiBase,
        sync_client: SyncClient,
        async_client: AsyncClient,
    ) -> None:
        self.api_base = api_base
        self.sync_client = sync_client
        self.async_client = async_client

    @property
    def sub_url(self) -> str:
        return "passwords/existing_password"

    def reset(
        self,
        email: str,
        existing_password: str,
        new_password: str,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ) -> ResetResponse:
        """[Stytch docs](https://stytch.com/docs/api/password-existing-password-reset)

        Reset the user’s password using their existing password.
        """  # noqa

        payload: Dict[str, Any] = {
            "email": email,
            "existing_password": existing_password,
            "new_password": new_password,
        }

        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt
        if session_duration_minutes is not None:
            payload["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims is not None:
            payload["session_custom_claims"] = session_custom_claims

        url = self.api_base.route_with_sub_url(self.sub_url, "reset")

        res = self.sync_client.post(url, json=payload)
        return ResetResponse.from_json(res.response.status_code, res.json)

    async def reset_async(
        self,
        email: str,
        existing_password: str,
        new_password: str,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ) -> ResetResponse:
        """[Stytch docs](https://stytch.com/docs/api/password-existing-password-reset)

        Reset the user’s password using their existing password.
        """  # noqa

        payload: Dict[str, Any] = {
            "email": email,
            "existing_password": existing_password,
            "new_password": new_password,
        }

        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt
        if session_duration_minutes is not None:
            payload["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims is not None:
            payload["session_custom_claims"] = session_custom_claims

        url = self.api_base.route_with_sub_url(self.sub_url, "reset")

        res = await self.async_client.post(url, json=payload)
        return ResetResponse.from_json(res.response.status, res.json)



#!/usr/bin/env python3

from typing import List

from stytch.b2b.core.models import B2BStytchSession, Member
from stytch.core.models import ResponseBase


class GetResponse(ResponseBase):
    """Response fields beyond those defined in `ResponseBase`:

    - `sessions`: An array of B2BStytchSession objects
    """  # noqa

    member_sessions: List[B2BStytchSession]


class AuthenticateResponse(ResponseBase):
    """Response fields beyond those defined in `ResponseBase`:

    - `member_session`: The Session object.

    - `session_token`: A secret token for a given Stytch Session. Read more about Session_token in our Session management guide.

    - `session_jwt`: The JSON Web Token (JWT) for a given Stytch Session. Read more about session_token in our session management guide.

    - `member`: The Member object.
    """  # noqa

    member_session: B2BStytchSession
    session_token: str
    session_jwt: str
    member: Member


class RevokeResponse(ResponseBase):
    """Response fields beyond those defined in `ResponseBase`:

    (None)
    """  # noqa



#!/usr/bin/env python3

from typing import List

from stytch.b2b.core.models import Member, Organization
from stytch.core.models import ResponseBase, SearchResultsMetadata


class CreateResponse(ResponseBase):
    """Response fields beyond those defined in `ResponseBase`:

    - `organization`: The Organization object.
    """  # noqa

    organization: Organization


class GetResponse(ResponseBase):
    """Response fields beyond those defined in `ResponseBase`:

    - `organization`: The Organization object.
    """  # noqa

    organization: Organization


class UpdateResponse(ResponseBase):
    """Response fields beyond those defined in `ResponseBase`:

    - `organization`: The Organization object.
    """  # noqa

    organization: Organization


class DeleteResponse(ResponseBase):
    """Response fields beyond those defined in `ResponseBase`:

    - `organization_id`: Globally unique UUID that identifies a specific Organization. The organization_id is critical to perform operations on an Organization, so be sure to preserve this value.
    """  # noqa

    organization_id: str


class DeleteMemberResponse(ResponseBase):
    """Response fields beyond those defined in `ResponseBase`:

    - `member_id`: Globally unique UUID that identifies a specific Member in the Stytch API. The member_id is critical to perform operations on a Member, so be sure to preserve this value.
    """  # noqa

    member_id: str


class SearchResponse(ResponseBase):
    """Response fields beyond those defined in `ResponseBase`:

    - `organizations`: An array of Organization objects.

    - `results_metadata`: Metadata relevant to your search query.
    """  # noqa

    organizations: List[Organization]
    results_metadata: SearchResultsMetadata


class SearchMembersResponse(ResponseBase):
    """Response fields beyond those defined in `ResponseBase`:

    - `members`: An array of Member objects.

    - `results_metadata`: Metadata relevant to your search query.
    """  # noqa

    members: List[Member]
    results_metadata: SearchResultsMetadata



#!/usr/bin/env python3

from typing import Optional

from stytch.core.models import ResponseBase, StytchSession, User


class AuthenticateResponse(ResponseBase):
    """[Stytch docs](https://stytch.com/docs/api/authenticate-otp)"""  # noqa

    user_id: str
    user: User
    method_id: str
    reset_sessions: bool
    session_jwt: str
    session_token: str
    session: Optional[StytchSession]



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from typing import Any, Dict, Optional, Union

import pydantic

from stytch.core.api_base import ApiBase
from stytch.core.http.client import AsyncClient, SyncClient
from stytch.core.models import Name
from stytch.models.magic_links_email import (
    InviteResponse,
    LoginOrCreateResponse,
    RevokeInviteResponse,
    SendResponse,
)


class Email:
    def __init__(
        self,
        api_base: ApiBase,
        sync_client: SyncClient,
        async_client: AsyncClient,
    ) -> None:
        self.api_base = api_base
        self.sync_client = sync_client
        self.async_client = async_client

    @property
    def sub_url(self) -> str:
        return "magic_links/email"

    def send(
        self,
        email: str,
        login_magic_link_url: Optional[str] = None,
        signup_magic_link_url: Optional[str] = None,
        login_expiration_minutes: Optional[int] = None,
        signup_expiration_minutes: Optional[int] = None,
        locale: Optional[str] = None,
        attributes: Optional[Dict[str, str]] = None,
        code_challenge: Optional[str] = None,
        user_id: Optional[str] = None,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        login_template_id: Optional[str] = None,
        signup_template_id: Optional[str] = None,
    ) -> SendResponse:
        """[Stytch docs](https://stytch.com/docs/api/send-by-email)

        Send a magic link to an existing Stytch user using their email address. If you'd like to create a user and send them a magic link by email with one request, use our [log in or create endpoint](https://stytch.com/docs/api/log-in-or-create-user-by-email).

        Parameters:

        - `email`: The email of the user to send the invite magic link to.

        - `login_magic_link_url`: The url the user clicks from the login email magic link. This should be a url that your app receives and parses and subsequently send an API request to authenticate the magic link and log in the user. If this value is not passed, the default login redirect URL that you set in your Dashboard is used. If you have not set a default login redirect URL, an error is returned.

        - `signup_magic_link_url`: The url the user clicks from the sign-up email magic link. This should be a url that your app receives and parses and subsequently send an api request to authenticate the magic link and sign-up the user. If this value is not passed, the default sign-up redirect URL that you set in your Dashboard is used. If you have not set a default sign-up redirect URL, an error is returned.

        - `login_expiration_minutes`: Set the expiration for the login email magic link, in minutes. By default, it expires in 1 hour. The minimum expiration is 5 minutes and the maximum is 7 days (10080 mins).

        - `signup_expiration_minutes`: Set the expiration for the sign-up email magic link, in minutes. By default, it expires in 1 week. The minimum expiration is 5 minutes and the maximum is 7 days (10080 mins).

        - `locale`: Used to determine which language to use when sending the user this delivery method. Parameter is a two character [IETF BCP 47 language tag](https://www.w3.org/International/articles/language-tags/), e.g. `"en"` or `"es"`.

          Currently supported languages are English (en) and Spanish (es); if no value is provided, the copy defaults to English.

          Request more languages for support [here](https://docs.google.com/forms/d/e/1FAIpQLScZSpAu_m2AmLXRT3F3kap-s_mcV6UTBitYn6CdyWP0-o7YjQ/viewform?usp=sf_link)!

        - `attributes`: Provided attributes help with fraud detection.

        - `code_challenge`: A base64url encoded SHA256 hash of a one time secret used to validate that the request starts and ends on the same device.

        - `user_id`: The `user_id` of the user to associate the email with. This parameter allows you to associate a new email address with an existing Stytch User.

          Only include a `user_id` if the user in question already has an active and valid session at the time of the send operation; without this safety check a malicious user may use this as an account takeover vector.

        - `session_token`: The `session_token` belonging to the user that you wish to associate the email with.

        - `session_jwt`: The `session_jwt` belonging to the user that you wish to associate the email with.
        """  # noqa

        payload: Dict[str, Any] = {
            "email": email,
        }

        if login_magic_link_url is not None:
            payload["login_magic_link_url"] = login_magic_link_url
        if signup_magic_link_url is not None:
            payload["signup_magic_link_url"] = signup_magic_link_url
        if login_expiration_minutes is not None:
            payload["login_expiration_minutes"] = login_expiration_minutes
        if signup_expiration_minutes is not None:
            payload["signup_expiration_minutes"] = signup_expiration_minutes
        if locale is not None:
            payload["locale"] = locale
        if attributes is not None:
            payload["attributes"] = attributes
        if code_challenge is not None:
            payload["code_challenge"] = code_challenge
        if user_id is not None:
            payload["user_id"] = user_id
        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt
        if login_template_id is not None:
            payload["login_template_id"] = login_template_id
        if signup_template_id is not None:
            payload["signup_template_id"] = signup_template_id

        url = self.api_base.route_with_sub_url(self.sub_url, "send")

        res = self.sync_client.post(url, json=payload)
        return SendResponse.from_json(res.response.status_code, res.json)

    async def send_async(
        self,
        email: str,
        login_magic_link_url: Optional[str] = None,
        signup_magic_link_url: Optional[str] = None,
        login_expiration_minutes: Optional[int] = None,
        signup_expiration_minutes: Optional[int] = None,
        locale: Optional[str] = None,
        attributes: Optional[Dict[str, str]] = None,
        code_challenge: Optional[str] = None,
        user_id: Optional[str] = None,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        login_template_id: Optional[str] = None,
        signup_template_id: Optional[str] = None,
    ) -> SendResponse:
        """[Stytch docs](https://stytch.com/docs/api/send-by-email)

        Send a magic link to an existing Stytch user using their email address. If you'd like to create a user and send them a magic link by email with one request, use our [log in or create endpoint](https://stytch.com/docs/api/log-in-or-create-user-by-email).

        Parameters:

        - `email`: The email of the user to send the invite magic link to.

        - `login_magic_link_url`: The url the user clicks from the login email magic link. This should be a url that your app receives and parses and subsequently send an API request to authenticate the magic link and log in the user. If this value is not passed, the default login redirect URL that you set in your Dashboard is used. If you have not set a default login redirect URL, an error is returned.

        - `signup_magic_link_url`: The url the user clicks from the sign-up email magic link. This should be a url that your app receives and parses and subsequently send an api request to authenticate the magic link and sign-up the user. If this value is not passed, the default sign-up redirect URL that you set in your Dashboard is used. If you have not set a default sign-up redirect URL, an error is returned.

        - `login_expiration_minutes`: Set the expiration for the login email magic link, in minutes. By default, it expires in 1 hour. The minimum expiration is 5 minutes and the maximum is 7 days (10080 mins).

        - `signup_expiration_minutes`: Set the expiration for the sign-up email magic link, in minutes. By default, it expires in 1 week. The minimum expiration is 5 minutes and the maximum is 7 days (10080 mins).

        - `locale`: Used to determine which language to use when sending the user this delivery method. Parameter is a two character [IETF BCP 47 language tag](https://www.w3.org/International/articles/language-tags/), e.g. `"en"` or `"es"`.

          Currently supported languages are English (en) and Spanish (es); if no value is provided, the copy defaults to English.

          Request more languages for support [here](https://docs.google.com/forms/d/e/1FAIpQLScZSpAu_m2AmLXRT3F3kap-s_mcV6UTBitYn6CdyWP0-o7YjQ/viewform?usp=sf_link)!

        - `attributes`: Provided attributes help with fraud detection.

        - `code_challenge`: A base64url encoded SHA256 hash of a one time secret used to validate that the request starts and ends on the same device.

        - `user_id`: The `user_id` of the user to associate the email with. This parameter allows you to associate a new email address with an existing Stytch User.

          Only include a `user_id` if the user in question already has an active and valid session at the time of the send operation; without this safety check a malicious user may use this as an account takeover vector.

        - `session_token`: The `session_token` belonging to the user that you wish to associate the email with.

        - `session_jwt`: The `session_jwt` belonging to the user that you wish to associate the email with.
        """  # noqa

        payload: Dict[str, Any] = {
            "email": email,
        }

        if login_magic_link_url is not None:
            payload["login_magic_link_url"] = login_magic_link_url
        if signup_magic_link_url is not None:
            payload["signup_magic_link_url"] = signup_magic_link_url
        if login_expiration_minutes is not None:
            payload["login_expiration_minutes"] = login_expiration_minutes
        if signup_expiration_minutes is not None:
            payload["signup_expiration_minutes"] = signup_expiration_minutes
        if locale is not None:
            payload["locale"] = locale
        if attributes is not None:
            payload["attributes"] = attributes
        if code_challenge is not None:
            payload["code_challenge"] = code_challenge
        if user_id is not None:
            payload["user_id"] = user_id
        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt
        if login_template_id is not None:
            payload["login_template_id"] = login_template_id
        if signup_template_id is not None:
            payload["signup_template_id"] = signup_template_id

        url = self.api_base.route_with_sub_url(self.sub_url, "send")

        res = await self.async_client.post(url, json=payload)
        return SendResponse.from_json(res.response.status, res.json)

    def login_or_create(
        self,
        email: str,
        login_magic_link_url: Optional[str] = None,
        signup_magic_link_url: Optional[str] = None,
        login_expiration_minutes: Optional[int] = None,
        signup_expiration_minutes: Optional[int] = None,
        create_user_as_pending: Optional[bool] = None,
        locale: Optional[str] = None,
        login_template_id: Optional[str] = None,
        signup_template_id: Optional[str] = None,
    ) -> LoginOrCreateResponse:
        """[Stytch docs](https://stytch.com/docs/api/log-in-or-create-user-by-email)

        Send either a login or signup magic link to the user based on if the email is associated with a user already. A new or pending user will receive a signup magic link. An active user will receive a login magic link. For more information on how to control the status your users are created in see the `create_user_as_pending` flag.

        Parameters:

        - `email`: The email of the user to send the invite magic link to.

        - `login_magic_link_url`: The url the user clicks from the login email magic link. This should be a url that your app receives and parses and subsequently send an API request to authenticate the magic link and log in the user. If this value is not passed, the default login redirect URL that you set in your Dashboard is used. If you have not set a default login redirect URL, an error is returned.

        - `signup_magic_link_url`: The url the user clicks from the sign-up email magic link. This should be a url that your app receives and parses and subsequently send an api request to authenticate the magic link and sign-up the user. If this value is not passed, the default sign-up redirect URL that you set in your Dashboard is used. If you have not set a default sign-up redirect URL, an error is returned.

        - `login_expiration_minutes`: Set the expiration for the login email magic link, in minutes. By default, it expires in 1 hour. The minimum expiration is 5 minutes and the maximum is 7 days (10080 mins).

        - `signup_expiration_minutes`: Set the expiration for the sign-up email magic link, in minutes. By default, it expires in 1 week. The minimum expiration is 5 minutes and the maximum is 7 days (10080 mins).

        - `create_user_as_pending`: Flag for whether or not to save a user as pending vs active in Stytch. Defaults to false. If true, new users will be created with status pending in Stytch's backend. Their status will remain pending and they will continue to receive sign-up magic links until a magic link is authenticated for that user. If false, new users will be created with status active. They will receive a sign-up magic link for their first magic link but subsequent magic links will use the login email template, even if the user never authenticated their initial magic link.

        - `locale`: Used to determine which language to use when sending the user this delivery method. Parameter is a two character [IETF BCP 47 language tag](https://www.w3.org/International/articles/language-tags/), e.g. `"en"` or `"es"`.

          Currently supported languages are English (en) and Spanish (es); if no value is provided, the copy defaults to English.

          Request more languages for support [here](https://docs.google.com/forms/d/e/1FAIpQLScZSpAu_m2AmLXRT3F3kap-s_mcV6UTBitYn6CdyWP0-o7YjQ/viewform?usp=sf_link)!

        - `attributes`: Provided attributes help with fraud detection.

        - `code_challenge`: A base64url encoded SHA256 hash of a one time secret used to validate that the request starts and ends on the same device.
        """  # noqa

        payload: Dict[str, Any] = {
            "email": email,
        }

        if login_magic_link_url is not None:
            payload["login_magic_link_url"] = login_magic_link_url
        if signup_magic_link_url is not None:
            payload["signup_magic_link_url"] = signup_magic_link_url
        if login_expiration_minutes is not None:
            payload["login_expiration_minutes"] = login_expiration_minutes
        if signup_expiration_minutes is not None:
            payload["signup_expiration_minutes"] = signup_expiration_minutes
        if create_user_as_pending is not None:
            payload["create_user_as_pending"] = create_user_as_pending
        if locale is not None:
            payload["locale"] = locale
        if login_template_id is not None:
            payload["login_template_id"] = login_template_id
        if signup_template_id is not None:
            payload["signup_template_id"] = signup_template_id

        url = self.api_base.route_with_sub_url(self.sub_url, "login_or_create")

        res = self.sync_client.post(url, json=payload)
        return LoginOrCreateResponse.from_json(res.response.status_code, res.json)

    async def login_or_create_async(
        self,
        email: str,
        login_magic_link_url: Optional[str] = None,
        signup_magic_link_url: Optional[str] = None,
        login_expiration_minutes: Optional[int] = None,
        signup_expiration_minutes: Optional[int] = None,
        create_user_as_pending: Optional[bool] = None,
        locale: Optional[str] = None,
        login_template_id: Optional[str] = None,
        signup_template_id: Optional[str] = None,
    ) -> LoginOrCreateResponse:
        """[Stytch docs](https://stytch.com/docs/api/log-in-or-create-user-by-email)

        Send either a login or signup magic link to the user based on if the email is associated with a user already. A new or pending user will receive a signup magic link. An active user will receive a login magic link. For more information on how to control the status your users are created in see the `create_user_as_pending` flag.

        Parameters:

        - `email`: The email of the user to send the invite magic link to.

        - `login_magic_link_url`: The url the user clicks from the login email magic link. This should be a url that your app receives and parses and subsequently send an API request to authenticate the magic link and log in the user. If this value is not passed, the default login redirect URL that you set in your Dashboard is used. If you have not set a default login redirect URL, an error is returned.

        - `signup_magic_link_url`: The url the user clicks from the sign-up email magic link. This should be a url that your app receives and parses and subsequently send an api request to authenticate the magic link and sign-up the user. If this value is not passed, the default sign-up redirect URL that you set in your Dashboard is used. If you have not set a default sign-up redirect URL, an error is returned.

        - `login_expiration_minutes`: Set the expiration for the login email magic link, in minutes. By default, it expires in 1 hour. The minimum expiration is 5 minutes and the maximum is 7 days (10080 mins).

        - `signup_expiration_minutes`: Set the expiration for the sign-up email magic link, in minutes. By default, it expires in 1 week. The minimum expiration is 5 minutes and the maximum is 7 days (10080 mins).

        - `create_user_as_pending`: Flag for whether or not to save a user as pending vs active in Stytch. Defaults to false. If true, new users will be created with status pending in Stytch's backend. Their status will remain pending and they will continue to receive sign-up magic links until a magic link is authenticated for that user. If false, new users will be created with status active. They will receive a sign-up magic link for their first magic link but subsequent magic links will use the login email template, even if the user never authenticated their initial magic link.

        - `locale`: Used to determine which language to use when sending the user this delivery method. Parameter is a two character [IETF BCP 47 language tag](https://www.w3.org/International/articles/language-tags/), e.g. `"en"` or `"es"`.

          Currently supported languages are English (en) and Spanish (es); if no value is provided, the copy defaults to English.

          Request more languages for support [here](https://docs.google.com/forms/d/e/1FAIpQLScZSpAu_m2AmLXRT3F3kap-s_mcV6UTBitYn6CdyWP0-o7YjQ/viewform?usp=sf_link)!

        - `attributes`: Provided attributes help with fraud detection.

        - `code_challenge`: A base64url encoded SHA256 hash of a one time secret used to validate that the request starts and ends on the same device.
        """  # noqa

        payload: Dict[str, Any] = {
            "email": email,
        }

        if login_magic_link_url is not None:
            payload["login_magic_link_url"] = login_magic_link_url
        if signup_magic_link_url is not None:
            payload["signup_magic_link_url"] = signup_magic_link_url
        if login_expiration_minutes is not None:
            payload["login_expiration_minutes"] = login_expiration_minutes
        if signup_expiration_minutes is not None:
            payload["signup_expiration_minutes"] = signup_expiration_minutes
        if create_user_as_pending is not None:
            payload["create_user_as_pending"] = create_user_as_pending
        if locale is not None:
            payload["locale"] = locale
        if login_template_id is not None:
            payload["login_template_id"] = login_template_id
        if signup_template_id is not None:
            payload["signup_template_id"] = signup_template_id

        url = self.api_base.route_with_sub_url(self.sub_url, "login_or_create")

        res = await self.async_client.post(url, json=payload)
        return LoginOrCreateResponse.from_json(res.response.status, res.json)

    def invite(
        self,
        email: str,
        invite_magic_link_url: Optional[str] = None,
        invite_expiration_minutes: Optional[int] = None,
        name: Optional[Union[Name, Dict[str, str]]] = None,
        locale: Optional[str] = None,
        attributes: Optional[Dict[str, str]] = None,
        invite_template_id: Optional[str] = None,
    ) -> InviteResponse:
        """[Stytch docs](https://stytch.com/docs/api/invite-by-email)

        Create a pending user and send an invite magic link to the provided email. The user will be created with a pending status until they click the magic link in the invite email.

        Parameters:

        - `email`: The email of the user to send the invite magic link to.

        - `invite_magic_link_url`: The url the user clicks from the email magic link. This should be a url that your app receives and parses and subsequently send an api request to authenticate the magic link and log in the user. If this value is not passed, the default invite redirect URL that you set in your Dashboard is used. If you have not set a default invite redirect URL, an error is returned.

        - `invite_expiration_minutes`: Set the expiration for the email magic link, in minutes. By default, it expires in 1 hour. The minimum expiration is 5 minutes and the maximum is 7 days (10080 mins).

        - `name`: The name of the user. Each field in the name object is optional.

        - `locale`: Used to determine which language to use when sending the user this delivery method. Parameter is a two character [IETF BCP 47 language tag](https://www.w3.org/International/articles/language-tags/), e.g. `"en"` or `"es"`.

          Currently supported languages are English (en) and Spanish (es); if no value is provided, the copy defaults to English.

          Request more languages for support [here](https://docs.google.com/forms/d/e/1FAIpQLScZSpAu_m2AmLXRT3F3kap-s_mcV6UTBitYn6CdyWP0-o7YjQ/viewform?usp=sf_link)!

        - `attributes`: Provided attributes help with fraud detection.
        """  # noqa

        payload: Dict[str, Any] = {
            "email": email,
        }

        if invite_magic_link_url is not None:
            payload["invite_magic_link_url"] = invite_magic_link_url
        if invite_expiration_minutes is not None:
            payload["invite_expiration_minutes"] = invite_expiration_minutes
        if name is not None:
            payload["name"] = (
                name.dict() if isinstance(name, pydantic.BaseModel) else name
            )
        if locale is not None:
            payload["locale"] = locale
        if attributes is not None:
            payload["attributes"] = attributes
        if invite_template_id is not None:
            payload["invite_template_id"] = invite_template_id

        url = self.api_base.route_with_sub_url(self.sub_url, "invite")

        res = self.sync_client.post(url, json=payload)
        return InviteResponse.from_json(res.response.status_code, res.json)

    async def invite_async(
        self,
        email: str,
        invite_magic_link_url: Optional[str] = None,
        invite_expiration_minutes: Optional[int] = None,
        name: Optional[Union[Name, Dict[str, str]]] = None,
        locale: Optional[str] = None,
        attributes: Optional[Dict[str, str]] = None,
        invite_template_id: Optional[str] = None,
    ) -> InviteResponse:
        """[Stytch docs](https://stytch.com/docs/api/invite-by-email)

        Create a pending user and send an invite magic link to the provided email. The user will be created with a pending status until they click the magic link in the invite email.

        Parameters:

        - `email`: The email of the user to send the invite magic link to.

        - `invite_magic_link_url`: The url the user clicks from the email magic link. This should be a url that your app receives and parses and subsequently send an api request to authenticate the magic link and log in the user. If this value is not passed, the default invite redirect URL that you set in your Dashboard is used. If you have not set a default invite redirect URL, an error is returned.

        - `invite_expiration_minutes`: Set the expiration for the email magic link, in minutes. By default, it expires in 1 hour. The minimum expiration is 5 minutes and the maximum is 7 days (10080 mins).

        - `name`: The name of the user. Each field in the name object is optional.

        - `locale`: Used to determine which language to use when sending the user this delivery method. Parameter is a two character [IETF BCP 47 language tag](https://www.w3.org/International/articles/language-tags/), e.g. `"en"` or `"es"`.

          Currently supported languages are English (en) and Spanish (es); if no value is provided, the copy defaults to English.

          Request more languages for support [here](https://docs.google.com/forms/d/e/1FAIpQLScZSpAu_m2AmLXRT3F3kap-s_mcV6UTBitYn6CdyWP0-o7YjQ/viewform?usp=sf_link)!

        - `attributes`: Provided attributes help with fraud detection.
        """  # noqa

        payload: Dict[str, Any] = {
            "email": email,
        }

        if invite_magic_link_url is not None:
            payload["invite_magic_link_url"] = invite_magic_link_url
        if invite_expiration_minutes is not None:
            payload["invite_expiration_minutes"] = invite_expiration_minutes
        if name is not None:
            payload["name"] = (
                name.dict() if isinstance(name, pydantic.BaseModel) else name
            )
        if locale is not None:
            payload["locale"] = locale
        if attributes is not None:
            payload["attributes"] = attributes
        if invite_template_id is not None:
            payload["invite_template_id"] = invite_template_id

        url = self.api_base.route_with_sub_url(self.sub_url, "invite")

        res = await self.async_client.post(url, json=payload)
        return InviteResponse.from_json(res.response.status, res.json)

    def revoke_invite(
        self,
        email: str,
    ) -> RevokeInviteResponse:
        """[Stytch docs](https://stytch.com/docs/api/revoke-pending-invite)

        Revoke a pending invite based on the email provided.

        Parameters:

        - `email`: The email of the user to revoke invite for.
        """  # noqa

        payload: Dict[str, Any] = {
            "email": email,
        }

        url = self.api_base.route_with_sub_url(self.sub_url, "revoke_invite")

        res = self.sync_client.post(url, json=payload)
        return RevokeInviteResponse.from_json(res.response.status_code, res.json)

    async def revoke_invite_async(
        self,
        email: str,
    ) -> RevokeInviteResponse:
        """[Stytch docs](https://stytch.com/docs/api/revoke-pending-invite)

        Revoke a pending invite based on the email provided.

        Parameters:

        - `email`: The email of the user to revoke invite for.
        """  # noqa

        payload: Dict[str, Any] = {
            "email": email,
        }

        url = self.api_base.route_with_sub_url(self.sub_url, "revoke_invite")

        res = await self.async_client.post(url, json=payload)
        return RevokeInviteResponse.from_json(res.response.status, res.json)



#!/usr/bin/env python3

import warnings

from stytch.b2b.api.magic_links import MagicLinks
from stytch.b2b.api.organizations import Organizations
from stytch.b2b.api.sessions import Sessions
from stytch.core.api_base import ApiBase
from stytch.core.http.client import AsyncClient, SyncClient


class Client:
    """
    Stytch B2B API Python client.

    Learn more at https://stytch.com/docs
    """

    def __init__(
        self,
        project_id: str,
        secret: str,
        environment: str,
        suppress_warnings: bool = False,
    ):
        base_url = self._env_url(environment, suppress_warnings)
        api_base = ApiBase(base_url)
        sync_client = SyncClient(project_id, secret)
        async_client = AsyncClient(project_id, secret)

        self.magic_links = MagicLinks(api_base, sync_client, async_client)
        self.organizations = Organizations(api_base, sync_client, async_client)
        self.sessions = Sessions(api_base, sync_client, async_client)

    @classmethod
    def _env_url(cls, env: str, suppress_warnings: bool = False) -> str:
        """Resolve the base URL for the Stytch API environment."""

        # Supported production environments
        if env == "test":
            if not suppress_warnings:
                warnings.warn("Test version of Stytch not intended for production use")
            return "https://test.stytch.com/v1/b2b/"
        elif env == "live":
            return "https://api.stytch.com/v1/b2b/"

        return env



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from typing import Any, Dict, Optional

from stytch.core.api_base import ApiBase
from stytch.core.http.client import AsyncClient, SyncClient
from stytch.models.crypto_wallets import AuthenticateResponse, AuthenticateStartResponse


class CryptoWallets:
    def __init__(
        self,
        api_base: ApiBase,
        sync_client: SyncClient,
        async_client: AsyncClient,
    ) -> None:
        self.api_base = api_base
        self.sync_client = sync_client
        self.async_client = async_client

    @property
    def sub_url(self) -> str:
        return "crypto_wallets"

    def authenticate_start(
        self,
        crypto_wallet_address: str,
        crypto_wallet_type: str,
        user_id: Optional[str] = None,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
    ) -> AuthenticateStartResponse:
        """[Stytch docs](https://stytch.com/docs/api/crypto-wallet-authenticate-start)

        Initiate the authentication of a crypto wallet. After calling this endpoint, the user will need to sign a message containing only the returned `challenge` field.

        Parameters:

        - `crypto_wallet_type`: The type of wallet to authenticate. Currently `ethereum` and `solana` are supported. Wallets for any EVM-compatible chains (such as Polygon or BSC) are also supported and are grouped under the `ethereum` type.

        - `crypto_wallet_address`: The address to authenticate.

        - `user_id`: The `user_id` belonging to the user you wish to associate the address with. If no form of user association is provided and the address is not associated with an existing Stytch user in your Project, a new user will be created.

        - `session_token`: The `session_token` belonging to the user you wish to associate the address with. If no form of user association is provided and the address is not associated with an existing Stytch user in your Project, a new user will be created.

        - `session_jwt`: The `session_jwt` belonging to the user you wish to associate the address with. If no form of user association is provided and the address is not associated with an existing Stytch user in your Project, a new user will be created.
        """  # noqa

        payload: Dict[str, Any] = {
            "crypto_wallet_address": crypto_wallet_address,
            "crypto_wallet_type": crypto_wallet_type,
        }

        if user_id is not None:
            payload["user_id"] = user_id
        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt

        url = self.api_base.route_with_sub_url(self.sub_url, "authenticate/start")

        res = self.sync_client.post(url, json=payload)
        return AuthenticateStartResponse.from_json(res.response.status_code, res.json)

    async def authenticate_start_async(
        self,
        crypto_wallet_address: str,
        crypto_wallet_type: str,
        user_id: Optional[str] = None,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
    ) -> AuthenticateStartResponse:
        """[Stytch docs](https://stytch.com/docs/api/crypto-wallet-authenticate-start)

        Initiate the authentication of a crypto wallet. After calling this endpoint, the user will need to sign a message containing only the returned `challenge` field.

        Parameters:

        - `crypto_wallet_type`: The type of wallet to authenticate. Currently `ethereum` and `solana` are supported. Wallets for any EVM-compatible chains (such as Polygon or BSC) are also supported and are grouped under the `ethereum` type.

        - `crypto_wallet_address`: The address to authenticate.

        - `user_id`: The `user_id` belonging to the user you wish to associate the address with. If no form of user association is provided and the address is not associated with an existing Stytch user in your Project, a new user will be created.

        - `session_token`: The `session_token` belonging to the user you wish to associate the address with. If no form of user association is provided and the address is not associated with an existing Stytch user in your Project, a new user will be created.

        - `session_jwt`: The `session_jwt` belonging to the user you wish to associate the address with. If no form of user association is provided and the address is not associated with an existing Stytch user in your Project, a new user will be created.
        """  # noqa

        payload: Dict[str, Any] = {
            "crypto_wallet_address": crypto_wallet_address,
            "crypto_wallet_type": crypto_wallet_type,
        }

        if user_id is not None:
            payload["user_id"] = user_id
        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt

        url = self.api_base.route_with_sub_url(self.sub_url, "authenticate/start")

        res = await self.async_client.post(url, json=payload)
        return AuthenticateStartResponse.from_json(res.response.status, res.json)

    def authenticate(
        self,
        crypto_wallet_address: str,
        crypto_wallet_type: str,
        signature: str,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ) -> AuthenticateResponse:
        """[Stytch docs](https://stytch.com/docs/api/crypto-wallet-authenticate)

        Complete the authentication of a crypto wallet by passing the signature.

        Parameters:

        - `crypto_wallet_type`: The type of wallet to authenticate. Currently `ethereum` and `solana` are supported.

        - `crypto_wallet_address`: The address to authenticate.

        - `signature`: The signature from the message.

        - `session_custom_claims`: Add a custom claims map to the session being authenticated. Claims are only created if a session is initialized by providing a value in `session duration minutes`. Claims will be included on the session object and in the JWT. To update a key in an existing session, supply a new value. To delete a key, supply a null value

          Custom claims made with reserved claims ("iss", "sub", "aud", "exp", "nbf", "iat", "jti") will be ignored. Total custom claims size cannot exceed four kilobytes

        - `session_duration_minutes`: Set the session lifetime to be this many minutes from now. This will start a new session if one doesn't already exist, returning both an opaque `session_token` and `session_jwt` for this session. Remember that the `session_jwt` will have a fixed lifetime of five minutes regardless of the underlying session duration, and will need to be refreshed over time.

          This value must be a minimum of 5 and a maximum of 129600 minutes (90 days).

          If a `session_token` or `session_jwt` is provided then a successful authentication will continue to extend the session this many minutes.

          If the `session_duration_minutes` parameter is not specified, a Stytch session will not be created.

        - `session_jwt`: Reuse an existing session instead of creating a new one. If you provide us with a `session_jwt`, then we'll update the session represented by this JWT with this crypto wallet factor. If this `session_jwt` belongs to a different user than the one tied to the address, the `session_jwt` will be ignored. This endpoint will error if both `session_token` and `session_jwt` are provided.

        - `session_token`: Reuse an existing session instead of creating a new one. If you provide us with a `session_token`, then we'll update the session represented by this session token with this crypto wallet factor. If this `session_token` belongs to a different user than the one tied to the address, the `session_token` will be ignored. This endpoint will error if both session_token and `session_jwt` are provided.
        """  # noqa

        payload: Dict[str, Any] = {
            "crypto_wallet_address": crypto_wallet_address,
            "crypto_wallet_type": crypto_wallet_type,
            "signature": signature,
        }

        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt
        if session_duration_minutes is not None:
            payload["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims is not None:
            payload["session_custom_claims"] = session_custom_claims

        url = self.api_base.route_with_sub_url(self.sub_url, "authenticate")

        res = self.sync_client.post(url, json=payload)
        return AuthenticateResponse.from_json(res.response.status_code, res.json)

    async def authenticate_async(
        self,
        crypto_wallet_address: str,
        crypto_wallet_type: str,
        signature: str,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ) -> AuthenticateResponse:
        """[Stytch docs](https://stytch.com/docs/api/crypto-wallet-authenticate)

        Complete the authentication of a crypto wallet by passing the signature.

        Parameters:

        - `crypto_wallet_type`: The type of wallet to authenticate. Currently `ethereum` and `solana` are supported.

        - `crypto_wallet_address`: The address to authenticate.

        - `signature`: The signature from the message.

        - `session_custom_claims`: Add a custom claims map to the session being authenticated. Claims are only created if a session is initialized by providing a value in `session duration minutes`. Claims will be included on the session object and in the JWT. To update a key in an existing session, supply a new value. To delete a key, supply a null value

          Custom claims made with reserved claims ("iss", "sub", "aud", "exp", "nbf", "iat", "jti") will be ignored. Total custom claims size cannot exceed four kilobytes

        - `session_duration_minutes`: Set the session lifetime to be this many minutes from now. This will start a new session if one doesn't already exist, returning both an opaque `session_token` and `session_jwt` for this session. Remember that the `session_jwt` will have a fixed lifetime of five minutes regardless of the underlying session duration, and will need to be refreshed over time.

          This value must be a minimum of 5 and a maximum of 129600 minutes (90 days).

          If a `session_token` or `session_jwt` is provided then a successful authentication will continue to extend the session this many minutes.

          If the `session_duration_minutes` parameter is not specified, a Stytch session will not be created.

        - `session_jwt`: Reuse an existing session instead of creating a new one. If you provide us with a `session_jwt`, then we'll update the session represented by this JWT with this crypto wallet factor. If this `session_jwt` belongs to a different user than the one tied to the address, the `session_jwt` will be ignored. This endpoint will error if both `session_token` and `session_jwt` are provided.

        - `session_token`: Reuse an existing session instead of creating a new one. If you provide us with a `session_token`, then we'll update the session represented by this session token with this crypto wallet factor. If this `session_token` belongs to a different user than the one tied to the address, the `session_token` will be ignored. This endpoint will error if both session_token and `session_jwt` are provided.
        """  # noqa

        payload: Dict[str, Any] = {
            "crypto_wallet_address": crypto_wallet_address,
            "crypto_wallet_type": crypto_wallet_type,
            "signature": signature,
        }

        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt
        if session_duration_minutes is not None:
            payload["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims is not None:
            payload["session_custom_claims"] = session_custom_claims

        url = self.api_base.route_with_sub_url(self.sub_url, "authenticate")

        res = await self.async_client.post(url, json=payload)
        return AuthenticateResponse.from_json(res.response.status, res.json)



#!/usr/bin/env python3

from typing import Any, Dict, Optional

from stytch.core.models import ResponseBase, StytchSession, User


class CreateResponse(ResponseBase):
    """[Stytch docs](https://stytch.com/docs/api/password-create)"""  # noqa

    user_id: str
    email_id: str


class AuthenticateResponse(ResponseBase):
    """[Stytch docs](https://stytch.com/docs/api/password-authenticate)"""  # noqa

    user_id: str
    user: User
    session_jwt: str
    session_token: str
    session: Optional[StytchSession]


class StrengthCheckResponse(ResponseBase):
    """[Stytch docs](https://stytch.com/docs/api/password-strength-check)"""  # noqa

    valid_password: bool
    score: int
    breached_password: bool
    feedback: Dict[str, Any]
    strength_policy: str
    breach_detection_on_create: bool


class MigrateResponse(ResponseBase):
    """[Stytch docs](https://stytch.com/docs/api/password-migrate)"""  # noqa

    user_id: str
    email_id: str
    user_created: bool



#!/usr/bin/env python3

from __future__ import annotations

import datetime
from typing import Any, Dict, List, Optional

import pydantic


class Organization(pydantic.BaseModel):
    organization_id: str
    organization_name: str
    organization_slug: str
    organization_logo_url: str
    trusted_metadata: Dict[str, Any]
    sso_default_connection_id: Optional[str]
    sso_jit_provisioning_allowed_connections: List[str]
    sso_jit_provisioning: str
    email_allowed_domains: List[str]
    email_jit_provisioning: str
    email_invites: str


class SSORegistration(pydantic.BaseModel):
    connection_id: str
    identity_provider: str
    external_id: str
    registration_id: str
    sso_registration_attributes: Dict[str, Any]


class Member(pydantic.BaseModel):
    organization_id: str
    member_id: str
    email_address: str
    status: str
    name: str
    sso_registrations: List[SSORegistration]


class B2BStytchSession(pydantic.BaseModel):
    member_session_id: str
    member_id: str
    started_at: datetime.datetime
    last_accessed_at: datetime.datetime
    expires_at: datetime.datetime
    authentication_factors: List[Dict[str, Any]]
    custom_claims: Optional[Dict[str, Any]]



#!/usr/bin/env python3

from __future__ import annotations

import datetime
from typing import Any, Dict, List, Optional

import pydantic


class ResponseBase(pydantic.BaseModel):
    status_code: int
    # Optional because it won't be present in a 5XX error
    request_id: Optional[str] = None

    @classmethod
    def from_json(cls, status_code: int, json: Dict[str, Any]):
        try:
            return cls(**json)
        except pydantic.ValidationError:
            # We need to be careful in case this one *also* fails
            try:
                details = StytchErrorDetails(**json)
                # Don't raise from here because then we trigger our
                # own fallback exception handling!
            except Exception as e:
                details = StytchErrorDetails.from_unknown(status_code)
                raise StytchError(details) from e
            raise StytchError(details) from None

    @property
    def is_informational(self) -> bool:
        return 100 <= self.status_code < 200

    @property
    def is_success(self) -> bool:
        return 200 <= self.status_code < 300

    @property
    def is_redirection(self) -> bool:
        return 300 <= self.status_code < 400

    @property
    def is_client_error(self) -> bool:
        return 400 <= self.status_code < 500

    @property
    def is_server_error(self) -> bool:
        return 500 <= self.status_code < 600


class StytchErrorDetails(ResponseBase):
    error_type: Optional[str]
    error_message: str
    error_url: Optional[str]

    @classmethod
    def from_unknown(cls, status_code: int) -> StytchErrorDetails:
        return StytchErrorDetails(
            status_code=status_code,
            request_id=None,
            error_type=None,
            error_message="An unknown error occurred",
            error_url=None,
        )


class StytchError(Exception):
    def __init__(self, details: StytchErrorDetails) -> None:
        self.details = details

    def __repr__(self) -> str:
        return f"StytchError {{{self.details}}}"

    def __str__(self) -> str:
        return str(self.details)


class Name(pydantic.BaseModel):
    first_name: Optional[str] = None
    middle_name: Optional[str] = None
    last_name: Optional[str] = None


class Operand(pydantic.BaseModel):
    filter_name: str
    filter_value: Any


class SearchQuery(pydantic.BaseModel):
    operator: str
    operands: List[Operand]


class EmailFactor(pydantic.BaseModel):
    email_address: str
    email_id: str


class StytchSession(pydantic.BaseModel):
    attributes: Optional[Dict[str, str]]
    authentication_factors: List[Dict[str, Any]]
    custom_claims: Optional[Dict[str, Any]]
    expires_at: datetime.datetime
    last_accessed_at: datetime.datetime
    session_id: str
    started_at: datetime.datetime
    user_id: str


class SessionPtr(pydantic.BaseModel):
    session: Optional[StytchSession]


class OAuthSession(pydantic.BaseModel):
    stytch_session: Optional[SessionPtr]
    session_id: str
    user_id: str
    started_at: Optional[datetime.datetime]
    last_accessed_at: Optional[datetime.datetime]
    expires_at: Optional[datetime.datetime]
    attributes: Optional[Dict[str, str]]
    authentication_factors: List[Dict[str, Any]]

    @property
    def session(self) -> Optional[StytchSession]:
        """Convenience property to retrieve the inner StytchSession if it exists"""
        if self.stytch_session is not None and self.stytch_session.session is not None:
            return self.stytch_session.session
        return None


class TOTPInstance(pydantic.BaseModel):
    totp_id: str
    verified: bool


class TOTPInstanceWithRecoveryCodes(TOTPInstance):
    recovery_codes: List[str]


class Email(pydantic.BaseModel):
    email_id: str
    email: str
    verified: bool


class PhoneNumber(pydantic.BaseModel):
    phone_id: str
    phone_number: str
    verified: bool


class OAuthProvider(pydantic.BaseModel):
    oauth_user_registration_id: str
    provider_subject: str
    provider_type: str
    profile_picture_url: str
    locale: str


class WebAuthnRegistration(pydantic.BaseModel):
    webauthn_registration_id: str
    domain: str
    user_agent: str
    authenticator_type: Optional[str]
    verified: bool


class BiometricRegistration(pydantic.BaseModel):
    biometric_registration_id: str
    verified: bool


class CryptoWallet(pydantic.BaseModel):
    crypto_wallet_id: str
    crypto_wallet_address: str
    crypto_wallet_type: str
    verified: bool


class Password(pydantic.BaseModel):
    password_id: str
    requires_reset: bool


class User(pydantic.BaseModel):
    name: Optional[Name]
    user_id: str
    trusted_metadata: Optional[Dict[str, Any]]
    untrusted_metadata: Optional[Dict[str, Any]]
    emails: List[Email]
    phone_numbers: List[PhoneNumber]
    providers: List[OAuthProvider]
    webauthn_registrations: List[WebAuthnRegistration]
    biometric_registrations: List[BiometricRegistration]
    totps: List[TOTPInstance]
    crypto_wallets: List[CryptoWallet]
    password: Optional[Password]
    created_at: Optional[datetime.datetime]
    status: str


class SearchResultsMetadata(pydantic.BaseModel):
    next_cursor: Optional[str]
    total: int



#!/usr/bin/env python3


from stytch.core.models import ResponseBase


class SendResponse(ResponseBase):
    """[Stytch docs](https://stytch.com/docs/api/send-by-email)

    Response fields beyond those defined in `ResponseBase`:

    - `user_id`: Globally unique UUID that identifies a specific user in the Stytch API. The `user_id` critical to perform operations on a user in our API, like Get user, Delete user, etc, so be sure to preserve this value.

    - `email_id`: Globally unique UUID that identifies a specific email address in the Stytch API. The `email_id` is used when you need to operate on a specific user's email address, e.g. to delete the email address from the Stytch user.
    """  # noqa

    user_id: str
    email_id: str


class LoginOrCreateResponse(ResponseBase):
    """[Stytch docs](https://stytch.com/docs/api/log-in-or-create-user-by-email)

    Response fields beyond those defined in `ResponseBase`:

    - `user_id`: Globally unique UUID that identifies a specific user in the Stytch API. The `user_id` critical to perform operations on a user in our API, like Get user, Delete user, etc, so be sure to preserve this value.

    - `email_id`: Globally unique UUID that identifies a specific email address in the Stytch API. The `email_id` is used when you need to operate on a specific user's email address, e.g. to delete the email address from the Stytch user.

    - `user_created`: In `login_or_create` endpoints, this field indicates whether or not a user was freshly created or the user went through a login path.
    """  # noqa

    user_id: str
    email_id: str
    user_created: bool


class InviteResponse(ResponseBase):
    """[Stytch docs](https://stytch.com/docs/api/invite-by-email)

    Response fields beyond those defined in `ResponseBase`:

    - `user_id`: Globally unique UUID that identifies a specific user in the Stytch API. The `user_id` critical to perform operations on a user in our API, like Get user, Delete user, etc, so be sure to preserve this value.

    - `email_id`: Globally unique UUID that identifies a specific email address in the Stytch API. The `email_id` is used when you need to operate on a specific user's email address, e.g. to delete the email address from the Stytch user.
    """  # noqa

    user_id: str
    email_id: str


class RevokeInviteResponse(ResponseBase):
    """[Stytch docs](https://stytch.com/docs/api/revoke-pending-invite)

    There are no response fields beyond those defined in `ResponseBase`
    """  # noqa



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from typing import Any, Dict, Optional

from stytch.b2b.models.magic_links_email import InviteResponse, LoginOrSignupResponse
from stytch.core.api_base import ApiBase
from stytch.core.http.client import AsyncClient, SyncClient


class Email:
    def __init__(
        self,
        api_base: ApiBase,
        sync_client: SyncClient,
        async_client: AsyncClient,
    ) -> None:
        self.api_base = api_base
        self.sync_client = sync_client
        self.async_client = async_client

    @property
    def sub_url(self) -> str:
        return "magic_links/email"

    def login_or_signup(
        self,
        organization_id: str,
        email_address: str,
        create_member_as_pending: Optional[bool] = None,
        login_redirect_url: Optional[str] = None,
        signup_redirect_url: Optional[str] = None,
        pkce_code_challenge: Optional[str] = None,
        login_template_id: Optional[str] = None,
        signup_template_id: Optional[str] = None,
    ) -> LoginOrSignupResponse:
        """Send either a login or signup magic link to a Member. A new or pending Member will receive a signup Email Magic Link. An active Member will receive a login Email Magic Link.

        Parameters:

        - `organization_id`: Globally unique UUID that identifies a specific Organization. The organization_id is critical to perform operations on an Organization, so be sure to preserve this value.

        - `email_address`: The email of the Member who will receive the Email Magic Link for login or signup.

        - `create_member_as_pending`: Flag for whether or not to save a Member as pending vs active in Stytch. Defaults to false. If true, new Members will be created with status pending in Stytch's backend. Their status will remain pending and they will continue to receive sign-up magic links until a magic link is authenticated for that Member. If false, new Members will be created with status active. They will receive a sign-up magic link for their first magic link but subsequent magic links will use the login email template, even if the Member never authenticated their initial magic link.

        - `login_redirect_url`: The URL that Member clicks from the login email magic link. This URL should be an endpoint in the backend server that verifies the request by querying Stytch's authenticate endpoint and finishes the login. If this value is not passed, the default login redirect URL that you set in your Dashboard is used. If you have not set a default login redirect URL, an error is returned.

        - `signup_redirect_url`: The url the Member clicks from the sign-up email magic link. This URL should be an endpoint in the backend server that verifies the request by querying Stytch's authenticate endpoint and finishes the login. If this value is not passed, the default sign-up redirect URL that you set in your Dashboard is used. If you have not set a default sign-up redirect URL, an error is returned.

        - `pkce_code_challenge`: A base64url encoded SHA256 hash of a one time secret used to validate that the request starts and ends on the same device. See the PKCE OAuth guide for usage instructions.

        - `login_template_id`: Use a custom template for login emails. By default, it will use your default email template The template must be a template using our built-in customizations or a custom HTML email for Magic links - Login.

        - `signup_template_id`: Use a custom template for sign-up emails. By default, it will use your default email template. The template must be a template using our built-in customizations or a custom HTML email for Magic links - Sign-up.
        """  # noqa

        payload: Dict[str, Any] = {
            "organization_id": organization_id,
            "email_address": email_address,
        }

        if create_member_as_pending is not None:
            payload["create_member_as_pending"] = create_member_as_pending
        if login_redirect_url is not None:
            payload["login_redirect_url"] = login_redirect_url
        if signup_redirect_url is not None:
            payload["signup_redirect_url"] = signup_redirect_url
        if pkce_code_challenge is not None:
            payload["pkce_code_challenge"] = pkce_code_challenge
        if login_template_id is not None:
            payload["login_template_id"] = login_template_id
        if signup_template_id is not None:
            payload["signup_template_id"] = signup_template_id

        url = self.api_base.route_with_sub_url(self.sub_url, "login_or_signup")

        res = self.sync_client.post(url, json=payload)
        return LoginOrSignupResponse.from_json(res.response.status_code, res.json)

    async def login_or_signup_async(
        self,
        organization_id: str,
        email_address: str,
        create_member_as_pending: Optional[bool] = None,
        login_redirect_url: Optional[str] = None,
        signup_redirect_url: Optional[str] = None,
        pkce_code_challenge: Optional[str] = None,
        login_template_id: Optional[str] = None,
        signup_template_id: Optional[str] = None,
    ) -> LoginOrSignupResponse:
        """Send either a login or signup magic link to a Member. A new or pending Member will receive a signup Email Magic Link. An active Member will receive a login Email Magic Link.

        Parameters:

        - `organization_id`: Globally unique UUID that identifies a specific Organization. The organization_id is critical to perform operations on an Organization, so be sure to preserve this value.

        - `email_address`: The email of the Member who will receive the Email Magic Link for login or signup.

        - `create_member_as_pending`: Flag for whether or not to save a Member as pending vs active in Stytch. Defaults to false. If true, new Members will be created with status pending in Stytch's backend. Their status will remain pending and they will continue to receive sign-up magic links until a magic link is authenticated for that Member. If false, new Members will be created with status active. They will receive a sign-up magic link for their first magic link but subsequent magic links will use the login email template, even if the Member never authenticated their initial magic link.

        - `login_redirect_url`: The URL that Member clicks from the login email magic link. This URL should be an endpoint in the backend server that verifies the request by querying Stytch's authenticate endpoint and finishes the login. If this value is not passed, the default login redirect URL that you set in your Dashboard is used. If you have not set a default login redirect URL, an error is returned.

        - `signup_redirect_url`: The url the Member clicks from the sign-up email magic link. This URL should be an endpoint in the backend server that verifies the request by querying Stytch's authenticate endpoint and finishes the login. If this value is not passed, the default sign-up redirect URL that you set in your Dashboard is used. If you have not set a default sign-up redirect URL, an error is returned.

        - `pkce_code_challenge`: A base64url encoded SHA256 hash of a one time secret used to validate that the request starts and ends on the same device. See the PKCE OAuth guide for usage instructions.

        - `login_template_id`: Use a custom template for login emails. By default, it will use your default email template The template must be a template using our built-in customizations or a custom HTML email for Magic links - Login.

        - `signup_template_id`: Use a custom template for sign-up emails. By default, it will use your default email template. The template must be a template using our built-in customizations or a custom HTML email for Magic links - Sign-up.
        """  # noqa

        payload: Dict[str, Any] = {
            "organization_id": organization_id,
            "email_address": email_address,
        }

        if create_member_as_pending is not None:
            payload["create_member_as_pending"] = create_member_as_pending
        if login_redirect_url is not None:
            payload["login_redirect_url"] = login_redirect_url
        if signup_redirect_url is not None:
            payload["signup_redirect_url"] = signup_redirect_url
        if pkce_code_challenge is not None:
            payload["pkce_code_challenge"] = pkce_code_challenge
        if login_template_id is not None:
            payload["login_template_id"] = login_template_id
        if signup_template_id is not None:
            payload["signup_template_id"] = signup_template_id

        url = self.api_base.route_with_sub_url(self.sub_url, "login_or_signup")

        res = await self.async_client.post(url, json=payload)
        return LoginOrSignupResponse.from_json(res.response.status, res.json)

    def invite(
        self,
        organization_id: str,
        email_address: str,
        invite_redirect_url: Optional[str] = None,
        invite_by_member_id: Optional[str] = None,
        name: Optional[str] = None,
        trusted_metadata: Optional[Dict[str, Any]] = None,
        untrusted_metadata: Optional[Dict[str, Any]] = None,
        invite_template_id: Optional[str] = None,
    ) -> InviteResponse:
        """Send an invite email to a new Member to join an Organization. The Member will be created with a pending status until they successfully authenticate.

        Parameters:

        - `organization_id`: Globally unique UUID that identifies a specific Organization in the Stytch API. The organization_id is critical to perform operations on an Organization so be sure to preserve this value.

        - `email_address`: The email address of the Member.

        - `invite_redirect_url`: The URL that Members click from the login email magic link. This URL should be an endpoint in the backend server that verifies the request by querying Stytch's authenticate endpoint and finishes the login. If this value is not passed, the default login redirect URL that you set in your Dashboard is used. If you have not set a default login redirect URL, an error is returned.

        - `invite_by_member_id`: The member_id of the Member who sent the invite.

        - `name`: The name of the invited Member.

        - `trusted_metadata`: An arbitrary JSON object for storing application-specific or identity-provider-specific data.

        - `untrusted_metadata`: The untrusted_metadata field contains an arbitrary JSON object of application-specific data. Untrusted metadata can be edited by end Members directly via the SDK, and cannot be used to store critical information. See the Metadata reference for complete field behavior details.

        - `invite_template_id`: Use a custom template for invite emails. By default, it will use your default email template. The template must be a template using our built-in customizations or a custom HTML email for Magic links - Invite.
        """  # noqa

        payload: Dict[str, Any] = {
            "organization_id": organization_id,
            "email_address": email_address,
        }

        if invite_redirect_url is not None:
            payload["invite_redirect_url"] = invite_redirect_url
        if invite_by_member_id is not None:
            payload["invite_by_member_id"] = invite_by_member_id
        if name is not None:
            payload["name"] = name
        if trusted_metadata is not None:
            payload["trusted_metadata"] = trusted_metadata
        if untrusted_metadata is not None:
            payload["untrusted_metadata"] = untrusted_metadata
        if invite_template_id is not None:
            payload["invite_template_id"] = invite_template_id

        url = self.api_base.route_with_sub_url(self.sub_url, "invite")

        res = self.sync_client.post(url, json=payload)
        return InviteResponse.from_json(res.response.status_code, res.json)

    async def invite_async(
        self,
        organization_id: str,
        email_address: str,
        invite_redirect_url: Optional[str] = None,
        invite_by_member_id: Optional[str] = None,
        name: Optional[str] = None,
        trusted_metadata: Optional[Dict[str, Any]] = None,
        untrusted_metadata: Optional[Dict[str, Any]] = None,
        invite_template_id: Optional[str] = None,
    ) -> InviteResponse:
        """Send an invite email to a new Member to join an Organization. The Member will be created with a pending status until they successfully authenticate.

        Parameters:

        - `organization_id`: Globally unique UUID that identifies a specific Organization in the Stytch API. The organization_id is critical to perform operations on an Organization so be sure to preserve this value.

        - `email_address`: The email address of the Member.

        - `invite_redirect_url`: The URL that Members click from the login email magic link. This URL should be an endpoint in the backend server that verifies the request by querying Stytch's authenticate endpoint and finishes the login. If this value is not passed, the default login redirect URL that you set in your Dashboard is used. If you have not set a default login redirect URL, an error is returned.

        - `invite_by_member_id`: The member_id of the Member who sent the invite.

        - `name`: The name of the invited Member.

        - `trusted_metadata`: An arbitrary JSON object for storing application-specific or identity-provider-specific data.

        - `untrusted_metadata`: The untrusted_metadata field contains an arbitrary JSON object of application-specific data. Untrusted metadata can be edited by end Members directly via the SDK, and cannot be used to store critical information. See the Metadata reference for complete field behavior details.

        - `invite_template_id`: Use a custom template for invite emails. By default, it will use your default email template. The template must be a template using our built-in customizations or a custom HTML email for Magic links - Invite.
        """  # noqa

        payload: Dict[str, Any] = {
            "organization_id": organization_id,
            "email_address": email_address,
        }

        if invite_redirect_url is not None:
            payload["invite_redirect_url"] = invite_redirect_url
        if invite_by_member_id is not None:
            payload["invite_by_member_id"] = invite_by_member_id
        if name is not None:
            payload["name"] = name
        if trusted_metadata is not None:
            payload["trusted_metadata"] = trusted_metadata
        if untrusted_metadata is not None:
            payload["untrusted_metadata"] = untrusted_metadata
        if invite_template_id is not None:
            payload["invite_template_id"] = invite_template_id

        url = self.api_base.route_with_sub_url(self.sub_url, "invite")

        res = await self.async_client.post(url, json=payload)
        return InviteResponse.from_json(res.response.status, res.json)



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from typing import Any, Dict, Optional

from stytch.api.magic_links_email import Email
from stytch.core.api_base import ApiBase
from stytch.core.http.client import AsyncClient, SyncClient
from stytch.models.magic_links import AuthenticateResponse, CreateResponse


class MagicLinks:
    def __init__(
        self,
        api_base: ApiBase,
        sync_client: SyncClient,
        async_client: AsyncClient,
    ) -> None:
        self.api_base = api_base
        self.sync_client = sync_client
        self.async_client = async_client
        self.email = Email(api_base, sync_client, async_client)

    @property
    def sub_url(self) -> str:
        return "magic_links"

    def create(
        self,
        user_id: str,
        expiration_minutes: Optional[int] = None,
        attributes: Optional[Dict[str, str]] = None,
    ) -> CreateResponse:
        """[Stytch docs](https://stytch.com/docs/api/create-magic-link)

        Create a magic link token for a user. Access to this endpoint is restricted, to be enabled for it, please send us a note at support@stytch.com.

        Parameters:

        - `user_id`: The user ID for the magic link token.

        - `expiration_minutes`: Set the expiration for the magic token, in minutes. By default, it expires in 1 hour. The minimum expiration is 5 minutes and the maximum is 7 days (10080 mins).

        - `attributes`: Provided attributes help with fraud detection. When authenticating a user's magic link token, you can require the IP address and/or the user agent to match that user's attributes when they originated the initial authentication request. To enable this functionality, you can use the options parameter in [AuthenticateMagic](https://stytch.com/docs/api/authenticate-magic-link).
        """  # noqa

        payload: Dict[str, Any] = {
            "user_id": user_id,
        }

        if expiration_minutes is not None:
            payload["expiration_minutes"] = expiration_minutes
        if attributes is not None:
            payload["attributes"] = attributes

        url = self.api_base.route_with_sub_url(self.sub_url, None)

        res = self.sync_client.post(url, json=payload)
        return CreateResponse.from_json(res.response.status_code, res.json)

    async def create_async(
        self,
        user_id: str,
        expiration_minutes: Optional[int] = None,
        attributes: Optional[Dict[str, str]] = None,
    ) -> CreateResponse:
        """[Stytch docs](https://stytch.com/docs/api/create-magic-link)

        Create a magic link token for a user. Access to this endpoint is restricted, to be enabled for it, please send us a note at support@stytch.com.

        Parameters:

        - `user_id`: The user ID for the magic link token.

        - `expiration_minutes`: Set the expiration for the magic token, in minutes. By default, it expires in 1 hour. The minimum expiration is 5 minutes and the maximum is 7 days (10080 mins).

        - `attributes`: Provided attributes help with fraud detection. When authenticating a user's magic link token, you can require the IP address and/or the user agent to match that user's attributes when they originated the initial authentication request. To enable this functionality, you can use the options parameter in [AuthenticateMagic](https://stytch.com/docs/api/authenticate-magic-link).
        """  # noqa

        payload: Dict[str, Any] = {
            "user_id": user_id,
        }

        if expiration_minutes is not None:
            payload["expiration_minutes"] = expiration_minutes
        if attributes is not None:
            payload["attributes"] = attributes

        url = self.api_base.route_with_sub_url(self.sub_url, None)

        res = await self.async_client.post(url, json=payload)
        return CreateResponse.from_json(res.response.status, res.json)

    def authenticate(
        self,
        token: str,
        attributes: Optional[Dict[str, str]] = None,
        options: Optional[Dict[str, str]] = None,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
        code_verifier: Optional[str] = None,
    ) -> AuthenticateResponse:
        """[Stytch docs](https://stytch.com/docs/api/authenticate-magic-link)

        Authenticate a user given a magic link. This endpoint verifies that the magic link `token` is valid, hasn't expired or been previously used, and any optional security settings such as IP match or user agent match are satisfied.

        Parameters:

        - `token`: The token to authenticate.

        - `options`: Specify optional security settings

        - `attributes`: Provided attributes help with fraud detection. You can require the IP address and/or the user agent to match the request used to send the magic link using the options parameter.

        - `session_duration_minutes`: Set the session lifetime to be this many minutes from now. This will start a new session if one doesn't already exist, returning both an opaque `session_token` and `session_jwt` for this session. Remember that the `session_jwt` will have a fixed lifetime of five minutes regardless of the underlying session duration, and will need to be refreshed over time.

          This value must be a minimum of 5 and a maximum of 129600 minutes (90 days).

          If a `session_token` or `session_jwt` is provided then a successful authentication will continue to extend the session this many minutes.

          If the `session_duration_minutes` parameter is not specified, a Stytch session will not be created.

        - `session_custom_claims`: Add a custom claims map to the session being authenticated. Claims are only created if a session is initialized by providing a value in `session_duration_minutes`. Claims will be included on the session object and in the JWT. To update a key in an existing session, supply a new value. To delete a key, supply a null value

          Custom claims made with reserved claims ("iss", "sub", "aud", "exp", "nbf", "iat", "jti") will be ignored. Total custom claims size cannot exceed four kilobytes

        - `session_jwt`: Reuse an existing session instead of creating a new one. If you provide us with a `session_jwt`, then we'll update the session represented by this JWT with this magic link factor. If this `session_jwt` belongs to a different user than the magic token, the `session_jwt` will be ignored. This endpoint will error if both `session_token` and `session_jwt` are provided.

        - `session_token`: Reuse an existing session instead of creating a new one. If you provide us with a `session_token`, then we'll update the session represented by this session token with this magic link factor. If this `session_token` belongs to a different user than the magic token, the `session_token` will be ignored. This endpoint will error if both `session_token` and `session_jwt` are provided.

        - `code_verifier`: A base64url encoded one time secret used to validate that the request starts and ends on the same device.
        """  # noqa

        payload: Dict[str, Any] = {
            "token": token,
        }

        if attributes is not None:
            payload["attributes"] = attributes
        if options is not None:
            payload["options"] = options
        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt
        if session_duration_minutes is not None:
            payload["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims is not None:
            payload["session_custom_claims"] = session_custom_claims
        if code_verifier is not None:
            payload["code_verifier"] = code_verifier

        url = self.api_base.route_with_sub_url(self.sub_url, "authenticate")

        res = self.sync_client.post(url, json=payload)
        return AuthenticateResponse.from_json(res.response.status_code, res.json)

    async def authenticate_async(
        self,
        token: str,
        attributes: Optional[Dict[str, str]] = None,
        options: Optional[Dict[str, str]] = None,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
        code_verifier: Optional[str] = None,
    ) -> AuthenticateResponse:
        """[Stytch docs](https://stytch.com/docs/api/authenticate-magic-link)

        Authenticate a user given a magic link. This endpoint verifies that the magic link `token` is valid, hasn't expired or been previously used, and any optional security settings such as IP match or user agent match are satisfied.

        Parameters:

        - `token`: The token to authenticate.

        - `options`: Specify optional security settings

        - `attributes`: Provided attributes help with fraud detection. You can require the IP address and/or the user agent to match the request used to send the magic link using the options parameter.

        - `session_duration_minutes`: Set the session lifetime to be this many minutes from now. This will start a new session if one doesn't already exist, returning both an opaque `session_token` and `session_jwt` for this session. Remember that the `session_jwt` will have a fixed lifetime of five minutes regardless of the underlying session duration, and will need to be refreshed over time.

          This value must be a minimum of 5 and a maximum of 129600 minutes (90 days).

          If a `session_token` or `session_jwt` is provided then a successful authentication will continue to extend the session this many minutes.

          If the `session_duration_minutes` parameter is not specified, a Stytch session will not be created.

        - `session_custom_claims`: Add a custom claims map to the session being authenticated. Claims are only created if a session is initialized by providing a value in `session_duration_minutes`. Claims will be included on the session object and in the JWT. To update a key in an existing session, supply a new value. To delete a key, supply a null value

          Custom claims made with reserved claims ("iss", "sub", "aud", "exp", "nbf", "iat", "jti") will be ignored. Total custom claims size cannot exceed four kilobytes

        - `session_jwt`: Reuse an existing session instead of creating a new one. If you provide us with a `session_jwt`, then we'll update the session represented by this JWT with this magic link factor. If this `session_jwt` belongs to a different user than the magic token, the `session_jwt` will be ignored. This endpoint will error if both `session_token` and `session_jwt` are provided.

        - `session_token`: Reuse an existing session instead of creating a new one. If you provide us with a `session_token`, then we'll update the session represented by this session token with this magic link factor. If this `session_token` belongs to a different user than the magic token, the `session_token` will be ignored. This endpoint will error if both `session_token` and `session_jwt` are provided.

        - `code_verifier`: A base64url encoded one time secret used to validate that the request starts and ends on the same device.
        """  # noqa

        payload: Dict[str, Any] = {
            "token": token,
        }

        if attributes is not None:
            payload["attributes"] = attributes
        if options is not None:
            payload["options"] = options
        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt
        if session_duration_minutes is not None:
            payload["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims is not None:
            payload["session_custom_claims"] = session_custom_claims
        if code_verifier is not None:
            payload["code_verifier"] = code_verifier

        url = self.api_base.route_with_sub_url(self.sub_url, "authenticate")

        res = await self.async_client.post(url, json=payload)
        return AuthenticateResponse.from_json(res.response.status, res.json)



#!/usr/bin/env python3


from stytch.b2b.core.models import Member
from stytch.core.models import ResponseBase


class LoginOrSignupResponse(ResponseBase):
    """Response fields beyond those defined in `ResponseBase`:

    - `member_id`: Globally unique UUID that identifies a specific Member in the Stytch API. The member_id is critical to perform operations on a Member, so be sure to preserve this value.

    - `member_created`: Boolean if a Member already exists with that email.

    - `member`: The Member object.
    """  # noqa

    member_id: str
    member_created: bool
    member: Member


class InviteResponse(ResponseBase):
    """Response fields beyond those defined in `ResponseBase`:

    - `member_id`: Globally unique UUID that identifies a specific Member. The member_id is critical to perform operations on a Member, so be sure to preserve this value.
    - `member`: The Member object.
    """  # noqa

    member_id: str
    member: Member



#!/usr/bin/env python3


from stytch.core.models import ResponseBase


class ResetStartResponse(ResponseBase):
    """[Stytch docs](https://stytch.com/docs/api/password-email-reset-start)"""  # noqa

    user_id: str
    email_id: str


class ResetResponse(ResponseBase):
    """[Stytch docs](https://stytch.com/docs/api/password-email-reset)"""  # noqa

    user_id: str



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from typing import Any, Dict, Optional

from stytch.core.api_base import ApiBase
from stytch.core.http.client import AsyncClient, SyncClient
from stytch.models.webauthn import (
    AuthenticateResponse,
    AuthenticateStartResponse,
    RegisterResponse,
    RegisterStartResponse,
)


class WebAuthn:
    def __init__(
        self,
        api_base: ApiBase,
        sync_client: SyncClient,
        async_client: AsyncClient,
    ) -> None:
        self.api_base = api_base
        self.sync_client = sync_client
        self.async_client = async_client

    @property
    def sub_url(self) -> str:
        return "webauthn"

    def register_start(
        self,
        user_id: str,
        domain: str,
        user_agent: Optional[str] = None,
        authenticator_type: Optional[str] = None,
    ) -> RegisterStartResponse:
        """[Stytch docs](https://stytch.com/docs/api/webauthn-register-start)

        Initiate the process of creating a new WebAuthn registration. After calling this endpoint, the browser will need to call [`navigator.credentials.create()`](https://www.w3.org/TR/webauthn-2/#sctn-createCredential) with the data from [`public_key_credential_creation_options`](https://w3c.github.io/webauthn/#dictionary-makecredentialoptions) passed to the [`navigator.credentials.create()`](https://www.w3.org/TR/webauthn-2/#sctn-createCredential) request via the public key argument. We recommend using the create()wrapper provided by the [webauthn-json](https://github.com/github/webauthn-json) library. If you are not using the webauthn-json library, the `public_key_credential_creation_options` will need to be converted to a suitable public key by unmarshalling the JSON, base64 decoding the user ID field, and converting user ID and the challenge fields into an array buffer.
        """  # noqa

        payload: Dict[str, Any] = {
            "user_id": user_id,
            "domain": domain,
        }

        if user_agent is not None:
            payload["user_agent"] = user_agent
        if authenticator_type is not None:
            payload["authenticator_type"] = authenticator_type

        url = self.api_base.route_with_sub_url(self.sub_url, "register/start")

        res = self.sync_client.post(url, json=payload)
        return RegisterStartResponse.from_json(res.response.status_code, res.json)

    async def register_start_async(
        self,
        user_id: str,
        domain: str,
        user_agent: Optional[str] = None,
        authenticator_type: Optional[str] = None,
    ) -> RegisterStartResponse:
        """[Stytch docs](https://stytch.com/docs/api/webauthn-register-start)

        Initiate the process of creating a new WebAuthn registration. After calling this endpoint, the browser will need to call [`navigator.credentials.create()`](https://www.w3.org/TR/webauthn-2/#sctn-createCredential) with the data from [`public_key_credential_creation_options`](https://w3c.github.io/webauthn/#dictionary-makecredentialoptions) passed to the [`navigator.credentials.create()`](https://www.w3.org/TR/webauthn-2/#sctn-createCredential) request via the public key argument. We recommend using the create()wrapper provided by the [webauthn-json](https://github.com/github/webauthn-json) library. If you are not using the webauthn-json library, the `public_key_credential_creation_options` will need to be converted to a suitable public key by unmarshalling the JSON, base64 decoding the user ID field, and converting user ID and the challenge fields into an array buffer.
        """  # noqa

        payload: Dict[str, Any] = {
            "user_id": user_id,
            "domain": domain,
        }

        if user_agent is not None:
            payload["user_agent"] = user_agent
        if authenticator_type is not None:
            payload["authenticator_type"] = authenticator_type

        url = self.api_base.route_with_sub_url(self.sub_url, "register/start")

        res = await self.async_client.post(url, json=payload)
        return RegisterStartResponse.from_json(res.response.status, res.json)

    def register(
        self,
        user_id: str,
        public_key_credential: str,
    ) -> RegisterResponse:
        """[Stytch docs](https://stytch.com/docs/api/webauthn-register)

        Complete the creation of a WebAuthn registration by passing the response from the [`navigator.credentials.create()`](https://www.w3.org/TR/webauthn-2/#sctn-createCredential) request to this endpoint as the `public_key_credential` parameter. If the [webauthn-json](https://github.com/github/webauthn-json) library's `create()` method was used, the response can be passed directly to the [register endpoint](https://stytch.com/docs/api/webauthn-register). If not, some fields (the client data and the attestation object) from the [`navigator.credentials.create()`](https://www.w3.org/TR/webauthn-2/#sctn-createCredential) response will need to be converted from array buffers to strings and marshalled into JSON.
        """  # noqa

        payload: Dict[str, Any] = {
            "user_id": user_id,
            "public_key_credential": public_key_credential,
        }

        url = self.api_base.route_with_sub_url(self.sub_url, "register")

        res = self.sync_client.post(url, json=payload)
        return RegisterResponse.from_json(res.response.status_code, res.json)

    async def register_async(
        self,
        user_id: str,
        public_key_credential: str,
    ) -> RegisterResponse:
        """[Stytch docs](https://stytch.com/docs/api/webauthn-register)

        Complete the creation of a WebAuthn registration by passing the response from the [`navigator.credentials.create()`](https://www.w3.org/TR/webauthn-2/#sctn-createCredential) request to this endpoint as the `public_key_credential` parameter. If the [webauthn-json](https://github.com/github/webauthn-json) library's `create()` method was used, the response can be passed directly to the [register endpoint](https://stytch.com/docs/api/webauthn-register). If not, some fields (the client data and the attestation object) from the [`navigator.credentials.create()`](https://www.w3.org/TR/webauthn-2/#sctn-createCredential) response will need to be converted from array buffers to strings and marshalled into JSON.
        """  # noqa

        payload: Dict[str, Any] = {
            "user_id": user_id,
            "public_key_credential": public_key_credential,
        }

        url = self.api_base.route_with_sub_url(self.sub_url, "register")

        res = await self.async_client.post(url, json=payload)
        return RegisterResponse.from_json(res.response.status, res.json)

    def authenticate_start(
        self,
        user_id: str,
        domain: str,
    ) -> AuthenticateStartResponse:
        """[Stytch docs](https://stytch.com/docs/api/webauthn-authenticate-start)

        Initiate the authentication of a WebAuthn registration. After calling this endpoint, the browser will need to call [`navigator.credentials.get()`](https://www.w3.org/TR/webauthn-2/#sctn-getAssertion) with the data from `public_key_credential_request_options` passed to the [`navigator.credentials.get()`](https://www.w3.org/TR/webauthn-2/#sctn-getAssertion) request via the public key argument. We recommend using the `get()` wrapper provided by the [webauthn-json](https://github.com/github/webauthn-json) library. If you are not using the webauthn-json library, the `public_key_credential_request_options` will need to be converted to a suitable public key by unmarshalling the JSON and converting some the fields to array buffers.
        """  # noqa

        payload: Dict[str, Any] = {
            "user_id": user_id,
            "domain": domain,
        }

        url = self.api_base.route_with_sub_url(self.sub_url, "authenticate/start")

        res = self.sync_client.post(url, json=payload)
        return AuthenticateStartResponse.from_json(res.response.status_code, res.json)

    async def authenticate_start_async(
        self,
        user_id: str,
        domain: str,
    ) -> AuthenticateStartResponse:
        """[Stytch docs](https://stytch.com/docs/api/webauthn-authenticate-start)

        Initiate the authentication of a WebAuthn registration. After calling this endpoint, the browser will need to call [`navigator.credentials.get()`](https://www.w3.org/TR/webauthn-2/#sctn-getAssertion) with the data from `public_key_credential_request_options` passed to the [`navigator.credentials.get()`](https://www.w3.org/TR/webauthn-2/#sctn-getAssertion) request via the public key argument. We recommend using the `get()` wrapper provided by the [webauthn-json](https://github.com/github/webauthn-json) library. If you are not using the webauthn-json library, the `public_key_credential_request_options` will need to be converted to a suitable public key by unmarshalling the JSON and converting some the fields to array buffers.
        """  # noqa

        payload: Dict[str, Any] = {
            "user_id": user_id,
            "domain": domain,
        }

        url = self.api_base.route_with_sub_url(self.sub_url, "authenticate/start")

        res = await self.async_client.post(url, json=payload)
        return AuthenticateStartResponse.from_json(res.response.status, res.json)

    def authenticate(
        self,
        public_key_credential: str,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ) -> AuthenticateResponse:
        """[Stytch docs](https://stytch.com/docs/api/webauthn-authenticate)

        Complete the authentication of a WebAuthn registration by passing the response from the [`navigator.credentials.get()`](https://www.w3.org/TR/webauthn-2/#sctn-getAssertion) request to the authenticate endpoint. If the [webauthn-json](https://github.com/github/webauthn-json) library's `get()` method was used, the response can be passed directly to the [authenticate endpoint](https://stytch.com/docs/api/webauthn-authenticate). If not some fields from the [`navigator.credentials.get()`](https://www.w3.org/TR/webauthn-2/#sctn-getAssertion) response will need to be converted from array buffers to strings and marshalled into JSON.
        """  # noqa

        payload: Dict[str, Any] = {
            "public_key_credential": public_key_credential,
        }

        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt
        if session_duration_minutes is not None:
            payload["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims is not None:
            payload["session_custom_claims"] = session_custom_claims

        url = self.api_base.route_with_sub_url(self.sub_url, "authenticate")

        res = self.sync_client.post(url, json=payload)
        return AuthenticateResponse.from_json(res.response.status_code, res.json)

    async def authenticate_async(
        self,
        public_key_credential: str,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ) -> AuthenticateResponse:
        """[Stytch docs](https://stytch.com/docs/api/webauthn-authenticate)

        Complete the authentication of a WebAuthn registration by passing the response from the [`navigator.credentials.get()`](https://www.w3.org/TR/webauthn-2/#sctn-getAssertion) request to the authenticate endpoint. If the [webauthn-json](https://github.com/github/webauthn-json) library's `get()` method was used, the response can be passed directly to the [authenticate endpoint](https://stytch.com/docs/api/webauthn-authenticate). If not some fields from the [`navigator.credentials.get()`](https://www.w3.org/TR/webauthn-2/#sctn-getAssertion) response will need to be converted from array buffers to strings and marshalled into JSON.
        """  # noqa

        payload: Dict[str, Any] = {
            "public_key_credential": public_key_credential,
        }

        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt
        if session_duration_minutes is not None:
            payload["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims is not None:
            payload["session_custom_claims"] = session_custom_claims

        url = self.api_base.route_with_sub_url(self.sub_url, "authenticate")

        res = await self.async_client.post(url, json=payload)
        return AuthenticateResponse.from_json(res.response.status, res.json)



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from typing import Any, Dict, Optional

from stytch.b2b.models.sessions import AuthenticateResponse, GetResponse, RevokeResponse
from stytch.core.api_base import ApiBase
from stytch.core.http.client import AsyncClient, SyncClient


class Sessions:
    def __init__(
        self,
        api_base: ApiBase,
        sync_client: SyncClient,
        async_client: AsyncClient,
    ) -> None:
        self.api_base = api_base
        self.sync_client = sync_client
        self.async_client = async_client

    @property
    def sub_url(self) -> str:
        return "sessions"

    def get(
        self,
        organization_id: str,
        member_id: str,
    ) -> GetResponse:
        """Retrieves all active Sessions for a Member.

        Parameters:

        - `organization_id`: Globally unique UUID that identifies a specific Organization in the Stytch API. The organization_id is critical to perform operations on an Organization, so be sure to preserve this value.

        - `member_id`: Globally unique UUID that identifies a specific Member. The member_id is critical to perform operations on a Member, so be sure to preserve this value.
        """  # noqa

        payload: Dict[str, Any] = {
            "organization_id": organization_id,
            "member_id": member_id,
        }

        url = self.api_base.route_with_sub_url(self.sub_url, None)

        res = self.sync_client.get(url, params=payload)
        return GetResponse.from_json(res.response.status_code, res.json)

    async def get_async(
        self,
        organization_id: str,
        member_id: str,
    ) -> GetResponse:
        """Retrieves all active Sessions for a Member.

        Parameters:

        - `organization_id`: Globally unique UUID that identifies a specific Organization in the Stytch API. The organization_id is critical to perform operations on an Organization, so be sure to preserve this value.

        - `member_id`: Globally unique UUID that identifies a specific Member. The member_id is critical to perform operations on a Member, so be sure to preserve this value.
        """  # noqa

        payload: Dict[str, Any] = {
            "organization_id": organization_id,
            "member_id": member_id,
        }

        url = self.api_base.route_with_sub_url(self.sub_url, None)

        res = await self.async_client.get(url, params=payload)
        return GetResponse.from_json(res.response.status, res.json)

    def authenticate(
        self,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ) -> AuthenticateResponse:
        """Authenticates a Session. You may provide a JWT that needs to be refreshed and is expired according to its exp claim. A new JWT will be returned If both the signature and the underlying Session are still valid.

        This endpoint requires only one of either the session_jwt or session_token be included in the request. It will return an error if both are present.

        Parameters:

        - `session_token`: A secret token for a given Stytch Session. Read more about session_token in our session management guide.

        - `session_jwt`: The JSON Web Token (JWT) for a given Stytch Session. Read more about session_token in our Session Management guide.

        - `session_duration_minutes`: The Session lifetime of this many minutes from now; minimum of 5 and a maximum of 129600 minutes (90 days). Note that a successful authentication will continue to extend the Session this many minutes.

        - `sessions_custom_claims`: Add a custom claims map to the Session being authenticated. Claims are only created if a Session is initialized by providing a value in session_duration_minutes. Claims will be included on the Session object and in the JWT. To update a key in an existing Session, supply a new value. To delete a key, supply a null value. Custom claims made with reserved claims ("iss", "sub", "aud", "exp", "nbf", "iat", "jti") will be ignored. Total custom claims size cannot exceed four kilobytes
        """  # noqa

        payload: Dict[str, Any] = {}

        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt
        if session_duration_minutes is not None:
            payload["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims is not None:
            payload["session_custom_claims"] = session_custom_claims

        url = self.api_base.route_with_sub_url(self.sub_url, "authenticate")

        res = self.sync_client.post(url, json=payload)
        return AuthenticateResponse.from_json(res.response.status_code, res.json)

    async def authenticate_async(
        self,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ) -> AuthenticateResponse:
        """Authenticates a Session. You may provide a JWT that needs to be refreshed and is expired according to its exp claim. A new JWT will be returned If both the signature and the underlying Session are still valid.

        This endpoint requires only one of either the session_jwt or session_token be included in the request. It will return an error if both are present.

        Parameters:

        - `session_token`: A secret token for a given Stytch Session. Read more about session_token in our session management guide.

        - `session_jwt`: The JSON Web Token (JWT) for a given Stytch Session. Read more about session_token in our Session Management guide.

        - `session_duration_minutes`: The Session lifetime of this many minutes from now; minimum of 5 and a maximum of 129600 minutes (90 days). Note that a successful authentication will continue to extend the Session this many minutes.

        - `sessions_custom_claims`: Add a custom claims map to the Session being authenticated. Claims are only created if a Session is initialized by providing a value in session_duration_minutes. Claims will be included on the Session object and in the JWT. To update a key in an existing Session, supply a new value. To delete a key, supply a null value. Custom claims made with reserved claims ("iss", "sub", "aud", "exp", "nbf", "iat", "jti") will be ignored. Total custom claims size cannot exceed four kilobytes
        """  # noqa

        payload: Dict[str, Any] = {}

        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt
        if session_duration_minutes is not None:
            payload["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims is not None:
            payload["session_custom_claims"] = session_custom_claims

        url = self.api_base.route_with_sub_url(self.sub_url, "authenticate")

        res = await self.async_client.post(url, json=payload)
        return AuthenticateResponse.from_json(res.response.status, res.json)

    def revoke(
        self,
        member_session_id: Optional[str] = None,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        member_id: Optional[str] = None,
    ) -> RevokeResponse:
        """Revoke a Session and immediately invalidate all its tokens. To revoke a specific Session, pass either the member_session_id, session_token, or session_jwt. To revoke all Sessions for a Member, pass the member_id. This endpoint requires exactly one body param to be included in the request. It will return an error if multiple are present.

        Parameters:

        - `member_session_id`: Globally unique UUID that identifies a specific Member’s Session.

        - `session_token`: A secret token for a given Stytch Session. Read more about session_token in our Session Management guide.

        - `session_jwt`: The JSON Web Token (JWT) for a given Stytch Session. Read more about session_token in our Session management guide.

        - `member_id`: Globally unique UUID that identifies a specific Member. The member_id is critical to perform operations on a Member, so be sure to preserve this value.
        """  # noqa

        payload: Dict[str, Any] = {}

        if member_session_id is not None:
            payload["member_session_id"] = member_session_id
        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt
        if member_id is not None:
            payload["member_id"] = member_id

        url = self.api_base.route_with_sub_url(self.sub_url, "revoke")

        res = self.sync_client.post(url, json=payload)
        return RevokeResponse.from_json(res.response.status_code, res.json)

    async def revoke_async(
        self,
        member_session_id: Optional[str] = None,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        member_id: Optional[str] = None,
    ) -> RevokeResponse:
        """Revoke a Session and immediately invalidate all its tokens. To revoke a specific Session, pass either the member_session_id, session_token, or session_jwt. To revoke all Sessions for a Member, pass the member_id. This endpoint requires exactly one body param to be included in the request. It will return an error if multiple are present.

        Parameters:

        - `member_session_id`: Globally unique UUID that identifies a specific Member’s Session.

        - `session_token`: A secret token for a given Stytch Session. Read more about session_token in our Session Management guide.

        - `session_jwt`: The JSON Web Token (JWT) for a given Stytch Session. Read more about session_token in our Session management guide.

        - `member_id`: Globally unique UUID that identifies a specific Member. The member_id is critical to perform operations on a Member, so be sure to preserve this value.
        """  # noqa

        payload: Dict[str, Any] = {}

        if member_session_id is not None:
            payload["member_session_id"] = member_session_id
        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt
        if member_id is not None:
            payload["member_id"] = member_id

        url = self.api_base.route_with_sub_url(self.sub_url, "revoke")

        res = await self.async_client.post(url, json=payload)
        return RevokeResponse.from_json(res.response.status, res.json)



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

import time
from typing import Any, Dict, Optional

import jwt

from stytch.core.api_base import ApiBase
from stytch.core.http.client import AsyncClient, SyncClient
from stytch.core.models import StytchSession
from stytch.models.sessions import (
    AuthenticateResponse,
    GetResponse,
    JwksResponse,
    RevokeResponse,
)


class Sessions:
    def __init__(
        self,
        api_base: ApiBase,
        sync_client: SyncClient,
        async_client: AsyncClient,
    ) -> None:
        self.api_base = api_base
        self.sync_client = sync_client
        self.async_client = async_client

    @property
    def sub_url(self) -> str:
        return "sessions"

    def get(
        self,
        user_id: str,
    ) -> GetResponse:
        """[Stytch docs](https://stytch.com/docs/api/session-get)

        List all active sessions for a given user ID. All timestamps are formatted according to the RFC 3339 standard and are expressed in UTC, e.g. `2021-12-29T12:33:09Z`.
        """  # noqa

        payload: Dict[str, Any] = {
            "user_id": user_id,
        }

        url = self.api_base.route_with_sub_url(self.sub_url, None)

        res = self.sync_client.get(url, params=payload)
        return GetResponse.from_json(res.response.status_code, res.json)

    async def get_async(
        self,
        user_id: str,
    ) -> GetResponse:
        """[Stytch docs](https://stytch.com/docs/api/session-get)

        List all active sessions for a given user ID. All timestamps are formatted according to the RFC 3339 standard and are expressed in UTC, e.g. `2021-12-29T12:33:09Z`.
        """  # noqa

        payload: Dict[str, Any] = {
            "user_id": user_id,
        }

        url = self.api_base.route_with_sub_url(self.sub_url, None)

        res = await self.async_client.get(url, params=payload)
        return GetResponse.from_json(res.response.status, res.json)

    def authenticate(
        self,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ) -> AuthenticateResponse:
        """[Stytch docs](https://stytch.com/docs/api/session-auth)

        Authenticate a session token and retrieve associated session data. If session_duration_minutes is included, update the lifetime of the session to be that many minutes from now. All timestamps are formatted according to the RFC 3339 standard and are expressed in UTC, e.g. `2021-12-29T12:33:09Z`

        This endpoint requires exactly one `session_jwt` or `session_token` as part of the request. If both are included you will receive a `too_many_session_arguments` error.
        """  # noqa

        payload: Dict[str, Any] = {}

        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt
        if session_duration_minutes is not None:
            payload["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims is not None:
            payload["session_custom_claims"] = session_custom_claims

        url = self.api_base.route_with_sub_url(self.sub_url, "authenticate")

        res = self.sync_client.post(url, json=payload)
        return AuthenticateResponse.from_json(res.response.status_code, res.json)

    async def authenticate_async(
        self,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ) -> AuthenticateResponse:
        """[Stytch docs](https://stytch.com/docs/api/session-auth)

        Authenticate a session token and retrieve associated session data. If session_duration_minutes is included, update the lifetime of the session to be that many minutes from now. All timestamps are formatted according to the RFC 3339 standard and are expressed in UTC, e.g. `2021-12-29T12:33:09Z`

        This endpoint requires exactly one `session_jwt` or `session_token` as part of the request. If both are included you will receive a `too_many_session_arguments` error.
        """  # noqa

        payload: Dict[str, Any] = {}

        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt
        if session_duration_minutes is not None:
            payload["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims is not None:
            payload["session_custom_claims"] = session_custom_claims

        url = self.api_base.route_with_sub_url(self.sub_url, "authenticate")

        res = await self.async_client.post(url, json=payload)
        return AuthenticateResponse.from_json(res.response.status, res.json)

    # MANUAL(authenticate_jwt)
    def authenticate_jwt(
        self,
        session_jwt: str,
        max_token_age_seconds: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ) -> AuthenticateResponse:
        """Parse a JWT and verify the signature, preferring local verification
        over remote.

        If max_token_age_seconds is set, remote verification will be forced if the
        JWT was issued at (based on the "iat" claim) more than that many seconds ago.

        To force remote validation for all tokens, set max_token_age_seconds to
        zero or use the authenticate method instead.

        NOTE: The response object will only have the user field set (ie, not None), if
        authentication is done remotely. Local authentication will *not* set the user
        field of the response object. In v7, it is planned for this method to directly
        return a StytchSession object to avoid this "partially set response" confusion.
        """
        # Return the local_result if available, otherwise call the Stytch API
        return self.authenticate_jwt_local(
            session_jwt=session_jwt,
            max_token_age_seconds=max_token_age_seconds,
        ) or self.authenticate(
            session_custom_claims=session_custom_claims, session_jwt=session_jwt
        )

    async def authenticate_jwt_async(
        self,
        session_jwt: str,
        max_token_age_seconds: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ) -> AuthenticateResponse:
        # Return the local_result if available, otherwise call the Stytch API
        return self.authenticate_jwt_local(
            session_jwt=session_jwt,
            max_token_age_seconds=max_token_age_seconds,
        ) or await self.authenticate_async(
            session_custom_claims=session_custom_claims, session_jwt=session_jwt
        )

    # ENDMANUAL(authenticate_jwt)

    # MANUAL(authenticate_jwt_local)
    def get_jwks_client(self) -> jwt.PyJWKClient:
        project_id = self.sync_client.project_id
        jwks_url = self.api_base.route_with_sub_url("sessions/jwks", project_id)
        return jwt.PyJWKClient(jwks_url)

    def authenticate_jwt_local(
        self,
        session_jwt: str,
        max_token_age_seconds: Optional[int] = None,
        leeway: int = 0,
    ) -> Optional[AuthenticateResponse]:
        """Parse a JWT and verify the signature locally
        (without calling /authenticate in the API).

        If max_token_age_seconds is set, this will return an error if the JWT was issued
        (based on the "iat" claim) more than maxTokenAge seconds ago.

        If max_token_age_seconds is explicitly set to zero, all tokens will be
        considered too old, even if they are otherwise valid.

        The value for leeway is the maximum allowable difference in seconds when
        comparing timestamps. It defaults to zero.

        NOTE: The response object will *never* have the user field set (ie, not None).
        Local authentication will *not* set the user field of the response object.
        In v7, it is planned for this method to directly return a StytchSession object
        to avoid this "partially set response" confusion.
        """
        project_id = self.sync_client.project_id
        jwt_audience = project_id
        jwt_issuer = "stytch.com/{}".format(project_id)
        _session_claim = "https://stytch.com/session"

        jwks_client = self.get_jwks_client()
        now = time.time()

        signing_key = jwks_client.get_signing_key_from_jwt(session_jwt)

        # NOTE: The max_token_age_seconds value is applied after decoding.
        payload = jwt.decode(
            session_jwt,
            signing_key.key,
            algorithms=["RS256"],
            options={
                "require": ["aud", "iss", "exp", "iat", "nbf"],
                "verify_signature": True,
                "verify_aud": True,
                "verify_iss": True,
                "verify_exp": True,
                "verify_iat": True,
                "verify_nbf": True,
            },
            audience=jwt_audience,
            issuer=jwt_issuer,
            leeway=leeway,
        )

        if max_token_age_seconds is not None:
            iat = payload["iat"]
            if now - iat >= max_token_age_seconds:
                # JWT was issued too long ago, don't verify
                return None

        # Unpack the session claim to match the detached session format.
        claim = payload[_session_claim]

        # For JWTs that include it, prefer the inner expires_at claim.
        expires_at = claim.get("expires_at", payload["exp"])

        session = StytchSession(
            attributes=claim["attributes"],
            authentication_factors=claim["authentication_factors"],
            expires_at=expires_at,
            last_accessed_at=claim["last_accessed_at"],
            session_id=claim["id"],
            started_at=claim["started_at"],
            user_id=payload["sub"],
            custom_claims=None,
        )

        return AuthenticateResponse(
            status_code=200,
            request_id="local-jwt-verification",
            session=session,
            session_jwt=session_jwt,
            session_token="",
            user=None,
        )

    # ENDMANUAL(authenticate_jwt_local)

    def revoke(
        self,
        session_id: Optional[str] = None,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
    ) -> RevokeResponse:
        """[Stytch docs](https://stytch.com/docs/api/session-revoke)

        Revoke a session, immediately invalidating all of its session tokens. You can revoke a session in three ways: using its ID, or using one of its session tokens, or one of its JWTs. This endpoint requires exactly one of those to be included in the request. It will return an error if multiple are present.
        """  # noqa

        payload: Dict[str, Any] = {}

        if session_id is not None:
            payload["session_id"] = session_id
        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt

        url = self.api_base.route_with_sub_url(self.sub_url, "revoke")

        res = self.sync_client.post(url, json=payload)
        return RevokeResponse.from_json(res.response.status_code, res.json)

    async def revoke_async(
        self,
        session_id: Optional[str] = None,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
    ) -> RevokeResponse:
        """[Stytch docs](https://stytch.com/docs/api/session-revoke)

        Revoke a session, immediately invalidating all of its session tokens. You can revoke a session in three ways: using its ID, or using one of its session tokens, or one of its JWTs. This endpoint requires exactly one of those to be included in the request. It will return an error if multiple are present.
        """  # noqa

        payload: Dict[str, Any] = {}

        if session_id is not None:
            payload["session_id"] = session_id
        if session_token is not None:
            payload["session_token"] = session_token
        if session_jwt is not None:
            payload["session_jwt"] = session_jwt

        url = self.api_base.route_with_sub_url(self.sub_url, "revoke")

        res = await self.async_client.post(url, json=payload)
        return RevokeResponse.from_json(res.response.status, res.json)

    def jwks(
        self,
        project_id: str,
    ) -> JwksResponse:
        """[Stytch docs](https://stytch.com/docs/api/jwks-get)

        Get the JSON Web Key Set (JWKS) for a project
        """  # noqa

        payload: Dict[str, Any] = {
            "project_id": project_id,
        }

        url = self.api_base.route_with_sub_url(self.sub_url, f"jwks/{project_id}")

        res = self.sync_client.get(url, params=payload)
        return JwksResponse.from_json(res.response.status_code, res.json)

    async def jwks_async(
        self,
        project_id: str,
    ) -> JwksResponse:
        """[Stytch docs](https://stytch.com/docs/api/jwks-get)

        Get the JSON Web Key Set (JWKS) for a project
        """  # noqa

        payload: Dict[str, Any] = {
            "project_id": project_id,
        }

        url = self.api_base.route_with_sub_url(self.sub_url, f"jwks/{project_id}")

        res = await self.async_client.get(url, params=payload)
        return JwksResponse.from_json(res.response.status, res.json)



#!/usr/bin/env python3

from typing import Optional

from stytch.b2b.core.models import B2BStytchSession, Member
from stytch.core.models import ResponseBase


class AuthenticateResponse(ResponseBase):
    """Response fields beyond those defined in `ResponseBase`:

    - `member_id`: Globally unique UUID that identifies a specific Member. The member_id is critical to perform operations on a Member, so be sure to preserve this value.

    - `method_id`: The email_id or phone_id involved in the given authentication.

    - `reset_sessions`: Indicates if all Sessions linked to the Member need to be reset. You should check this field if you aren't using Stytch's Session product. If you are using Stytch's Session product, we revoke the Member’s other Sessions for you.

    - `organization_id`: Globally unique UUID that identifies a specific Organization in the Stytch API. The organization_id is critical to perform operations on an Organization so be sure to preserve this value.

    - `member`: The Member object.

    - `session_token`: A secret token for a given Stytch Session. Read more about session_token in our Session management guide.

    - `session_jwt`: The JSON Web Token (JWT) for a given Stytch Session. Read more about session_token in our Session Management guide.

    - `member_session`: The B2BStytchSession object.
    """  # noqa

    member_id: str
    method_id: str
    reset_sessions: bool
    organization_id: str
    member: Member
    session_token: str
    session_jwt: str
    member_session: Optional[B2BStytchSession]
